"""Test the bluetooth_numbers.utils module."""
from uuid import UUID

import pytest

from bluetooth_numbers.exceptions import (
    No16BitIntegerError,
    NonStandardUUIDError,
    WrongOUIFormatError,
)
from bluetooth_numbers.utils import (
    is_normalized_oui,
    is_standard_uuid128,
    is_uint16,
    normalize_oui,
    uint16_to_hex,
    uuid16_to_uuid128,
    uuid128_to_uuid16,
)


@pytest.mark.parametrize(
    ("oui", "result"),
    [
        ("70:BC:10", True),
        ("44:ec:ce", False),
        ("00-80-C7", False),
        ("6C2B59", False),
        ("a41194", False),
        ("FOOBAR", False),
    ],
)
def test_is_normalized_oui(oui: str, result: bool) -> None:
    """Test whether the is_normalized_oui function works correctly.

    It should return True for a normalized OUI and False otherwise. It should also
    return False if the argument isn't even an unnormalized OUI.
    """
    assert is_normalized_oui(oui) == result


@pytest.mark.parametrize(
    ("oui", "normalized"),
    [
        ("00:1B:C0", "00:1B:C0"),
        ("c0:d0:ff", "C0:D0:FF"),
        ("80-7D-3A", "80:7D:3A"),
        ("10-94-bb", "10:94:BB"),
        ("7C6456", "7C:64:56"),
        ("ac6706", "AC:67:06"),
    ],
)
def test_normalize_oui(oui: str, normalized: str) -> None:
    """Test whether the normalize_oui function normalizes OUIs."""
    assert normalize_oui(oui) == normalized


@pytest.mark.parametrize(
    "oui",
    [
        "AB:CD:EF:GH:IJ:KL",
        "gg-hh-ii",
        "FOOBAR",
    ],
)
def test_normalize_oui_exceptions(oui: str) -> None:
    """Test the normalize_oui function with invalid arguments.

    It should raise a WrongOUIFormatError exception when the argument doesn't have
    the right format.
    """
    with pytest.raises(WrongOUIFormatError):
        normalize_oui(oui)


@pytest.mark.parametrize(
    ("number", "result"),
    [
        (0x0000, True),
        (0x1800, True),
        (0x0499, True),
        (0xFD6F, True),
        (-1, False),
        (65536, False),
        (6.5, False),
        ("true", False),
    ],
)
def test_is_uint16(number: int, result: bool) -> None:
    """Test whether the is_uint16 function works correctly.

    It should return True when a number is a 16-bit unsigned integer; False otherwise.
    """
    assert is_uint16(number) == result


@pytest.mark.parametrize(
    ("number", "hex_string"),
    [
        (0x004C, "0x004c"),
        (0x0499, "0x0499"),
        (0xFD6F, "0xfd6f"),
    ],
)
def test_uint16_to_hex(number: int, hex_string: str) -> None:
    """Test the uint16_to_hex function with 16-bit unsigned integers.

    It should convert a 16-bit unsigned integer to its string representation.
    """
    assert uint16_to_hex(number) == hex_string


@pytest.mark.parametrize(
    "number",
    [
        -1,
        65536,
        4.5,
    ],
)
def test_invalid_uint16_to_hex(number: int) -> None:
    """Test the uint16_to_hex function with invalid arguments.

    It should raise a No16BitIntegerError for an invalid argument.
    """
    with pytest.raises(No16BitIntegerError):
        uint16_to_hex(number)


@pytest.mark.parametrize(
    ("uuid16", "uuid128"),
    [
        (0x1800, UUID("00001800-0000-1000-8000-00805F9B34FB")),
        (0xFD6F, UUID("0000FD6F-0000-1000-8000-00805F9B34FB")),
    ],
)
def test_uuid16_to_uuid128(uuid16: int, uuid128: UUID) -> None:
    """Test the uuid16_to_uuid128 function with 16-bit UUIDs.

    It should convert a 16-bit UUID to a 128-bit UUID with the Bluetooth base UUID.
    """
    assert uuid16_to_uuid128(uuid16) == uuid128


@pytest.mark.parametrize(
    "uuid16",
    [
        -1,
        65536,
        4.5,
    ],
)
def test_invalid_uuid16_to_uuid128(uuid16: int) -> None:
    """Test the uuid16_to_uuid128 function with invalid arguments.

    It should raise a No16BitIntegerError for an invalid argument.
    """
    with pytest.raises(No16BitIntegerError):
        uuid16_to_uuid128(uuid16)


@pytest.mark.parametrize(
    ("uuid128", "standard"),
    [
        (UUID("00001800-0000-1000-8000-00805F9B34FB"), True),
        (UUID("0000FD6F-0000-1000-8000-00805F9B34FB"), True),
        (UUID("bfc46884-ea75-416b-8154-29c5d0b0a087"), False),
        (UUID("00001800-0000-1000-8000-00805F9B34FC"), False),
    ],
)
def test_is_standard_uuid128(uuid128: UUID, standard: bool) -> None:
    """Test the is_standard_uuid128 function.

    It should return ``True`` for a 128-bit standard Bluetooth UUID,
    ``False`` otherwise.
    """
    assert is_standard_uuid128(uuid128) == standard


@pytest.mark.parametrize(
    ("uuid128", "uuid16"),
    [
        (UUID("00001800-0000-1000-8000-00805F9B34FB"), 0x1800),
        (UUID("0000FD6F-0000-1000-8000-00805F9B34FB"), 0xFD6F),
    ],
)
def test_uuid128_to_uuid16(uuid128: UUID, uuid16: int) -> None:
    """Test the uuid128_to_uuid16 function with 128-bit standard UUIDs.

    It should convert a 128-bit standard Bluetooth UUID to a 16-bit UUID.
    """
    assert uuid128_to_uuid16(uuid128) == uuid16


@pytest.mark.parametrize(
    "uuid128",
    [
        UUID("bfc46884-ea75-416b-8154-29c5d0b0a087"),
        UUID("00001800-0000-1000-8000-00805F9B34FC"),
    ],
)
def test_invalid_uuid128_to_uuid16(uuid128: UUID) -> None:
    """Test the uuid128_to_uuid16 function with invalid arguments.

    It should raise a NonStandardUUIDError for an invalid argument.
    """
    with pytest.raises(NonStandardUUIDError):
        uuid128_to_uuid16(uuid128)



"""Generate Python modules for Bluetooth numbers."""
from __future__ import annotations

import json
import re
from pathlib import Path

from jinja2 import Environment, FileSystemLoader

DATA_DIR = "data"
BLUETOOTH_NUMBERS_DIR = f"{DATA_DIR}/bluetooth-numbers-database/v1"
TEMPLATE_DIR = "templates"
CODE_DIR = "src/bluetooth_numbers"
UUID_TEMPLATE = "uuids.py.jinja"
CIC_TEMPLATE = "companies.py.jinja"
OUI_TEMPLATE = "ouis.py.jinja"
OUI_RE = re.compile(r"^([0-9A-F]{2}-[0-9A-F]{2}-[0-9A-F]{2})\s*\(hex\)\s+(.*)\s*$")

file_loader = FileSystemLoader(TEMPLATE_DIR)
env = Environment(loader=file_loader, autoescape=False)  # noqa: S701


def generate_uuid16_dictionary(kind: str) -> dict[int, str]:
    """Generate 16-bit UUID dictionary for a module.

    Args:
        kind (str): Should be "sdo_service".

    Returns:
        dict[int, str]: A dict with 16-bit UUIDs as keys.
    """
    uuid16_dict = {}

    with (Path(DATA_DIR) / f"{kind}_uuids.json").open() as json_file:
        json_data = json.loads(json_file.read())
        for number in json_data:
            name = number["name"]
            uuid = number["uuid"]
            uuid16_dict[uuid] = name

    return uuid16_dict


def generate_uuid_dictionaries(kind: str) -> tuple[dict[int, str], dict[str, str]]:
    """Generate UUID dictionaries for a module.

    Args:
        kind (str): Should be "service", "characteristic", or "descriptor".

    Returns:
        tuple[dict[int, str], dict[str, str]]: A tuple of dicts. The first dict
          has 16-bit UUIDs as keys, the second one 128-bit UUIDs.
    """
    uuid16_dict = {}
    uuid128_dict = {}

    with (Path(BLUETOOTH_NUMBERS_DIR) / f"{kind}_uuids.json").open() as json_file:
        json_data = json.loads(json_file.read())
        for number in json_data:
            name = number["name"]
            uuid = number["uuid"]
            if len(uuid) == 4:  # noqa: PLR2004
                uuid16_dict[uuid] = name
            else:
                uuid128_dict[uuid] = name

    return uuid16_dict, uuid128_dict


def generate_uuid_module(
    kind: str,
    uuid16_dict: dict[int, str],
    uuid128_dict: dict[str, str],
) -> None:
    """Generate Python module for UUIDs.

    Args:
        kind (str): Should be "service", "characteristic", or "descriptor".
        uuid16_dict (dict[int, str]): Dict with 16-bit UUIDs as keys.
        uuid128_dict (dict[str, str]): Dict with 128-bit UUIDs as keys.
    """
    template = env.get_template(UUID_TEMPLATE)
    with (Path(CODE_DIR) / f"_{kind}s.py").open("w") as python_file:
        python_file.write(
            template.render(uuids16=uuid16_dict, uuids128=uuid128_dict, uuid_dict=kind),
        )


def generate_cic_dictionary() -> dict[str, str]:
    """Generate Company ID Code dictionary for a module.

    Returns:
        dict[str, str]: A dict with CIC keys and their name.
    """
    cic_dict = {}

    with (Path(BLUETOOTH_NUMBERS_DIR) / "company_ids.json").open() as json_file:
        json_data = json.loads(json_file.read())
        for number in json_data:
            code = f"{number['code']:#06x}"
            name = number["name"].replace('"', '\\"')
            cic_dict[code] = name

    return cic_dict


def generate_cic_module(cic_dict: dict[str, str]) -> None:
    """Generate Python module for Company ID Codes.

    Args:
        cic_dict (dict[str, str]): The dict with CICs to generate a Python
          module for.
    """
    template = env.get_template(CIC_TEMPLATE)
    with (Path(CODE_DIR) / "_companies.py").open("w") as python_file:
        python_file.write(template.render(cics=cic_dict))


def generate_oui_dictionary() -> dict[str, str]:
    """Generate OUI dictionary for a module.

    Returns:
        dict[str, str]: A dict with OUI prefixes and their name.
    """
    oui_dict = {}

    with (Path(DATA_DIR) / "oui.txt").open() as txt_file:
        for line in txt_file:
            extracted = OUI_RE.match(line)
            if extracted:
                oui_dict[extracted.group(1).replace("-", ":")] = extracted.group(2)

    return oui_dict


def generate_oui_module(oui_dict: dict[str, str]) -> None:
    """Generate Python module for OUIs.

    Args:
        oui_dict (dict[str, str]): The dict with OUIs to generate a Python
          module for.
    """
    template = env.get_template(OUI_TEMPLATE)
    with (Path(CODE_DIR) / "_ouis.py").open("w") as python_file:
        python_file.write(template.render(ouis=oui_dict))


if __name__ == "__main__":
    # Generate module for service UUIDs
    service_uuid16, service_uuid128 = generate_uuid_dictionaries("service")
    member_service_uuid16 = generate_uuid16_dictionary("member_service")
    sdo_service_uuid16 = generate_uuid16_dictionary("sdo_service")
    # Don't let the UUIDs from the Assigned Numbers document overwrite the ones from the
    # Bluetooth Numbers Database because the latter has the names of the services, which
    # give more information than the names of the companies in the Assigned Numbers
    # document.
    service_uuid16.update(
        {
            key: value
            for key, value in member_service_uuid16.items()
            if key not in service_uuid16
        },
    )
    service_uuid16.update(sdo_service_uuid16)
    generate_uuid_module("service", service_uuid16, service_uuid128)

    # Generate module for characteristic UUIDs
    characteristic_uuid16, characteristic_uuid128 = generate_uuid_dictionaries(
        "characteristic",
    )
    generate_uuid_module(
        "characteristic",
        characteristic_uuid16,
        characteristic_uuid128,
    )

    # Generate module for descriptor UUIDs
    descriptor_uuid16, descriptor_uuid128 = generate_uuid_dictionaries("descriptor")
    generate_uuid_module("descriptor", descriptor_uuid16, descriptor_uuid128)

    # Generate module for Company ID Codes
    cics = generate_cic_dictionary()
    generate_cic_module(cics)

    # Generate module for OUIs
    ouis = generate_oui_dictionary()
    generate_oui_module(ouis)



"""Test the bluetooth_numbers._ouis module."""
import pytest

from bluetooth_numbers import oui
from bluetooth_numbers.exceptions import UnknownOUIError, WrongOUIFormatError


@pytest.mark.parametrize(
    ("prefix", "name"),
    [
        ("58:2D:34", "Qingping Electronics (Suzhou) Co., Ltd"),
        ("C4:7C:8D", "IEEE Registration Authority"),
        ("ac:67:06", "Ruckus Wireless"),
        ("B8-75-C0", "PayPal, Inc."),
        ("245BA7", "Apple, Inc."),
        ("a0-6f-aa", "LG Innotek"),
        ("407c7d", "Nokia"),
    ],
)
def test_oui(prefix: str, name: str) -> None:
    """Test the oui dict with known values."""
    assert oui[prefix] == name


@pytest.mark.parametrize(
    "prefix",
    [
        "FOOBAR",
        "AB-CD-EF-GH",
    ],
)
def test_invalid_oui(prefix: str) -> None:
    """Test the oui dict with invalid prefixes.

    Using an invalid prefix as a key should raise a WrongOUIFormatError exception.
    """
    with pytest.raises(WrongOUIFormatError):
        _ = oui[prefix]


@pytest.mark.parametrize(
    "prefix",
    [
        "12:34:56",
        "AB:CD:EF",
    ],
)
def test_unknown_oui(prefix: str) -> None:
    """Test the oui dict with unknown prefixes.

    Using an unknown prefix as a key should raise an UnknownOUIError exception.
    """
    with pytest.raises(UnknownOUIError):
        _ = oui[prefix]



"""Test the bluetooth_numbers._characteristics module."""
from uuid import UUID

import pytest

from bluetooth_numbers import characteristic
from bluetooth_numbers.exceptions import No16BitIntegerError, UnknownUUIDError


@pytest.mark.parametrize(
    ("uuid", "name"),
    [
        (0x2A7E, "Aerobic Heart Rate Lower Limit"),
        (0x2A37, "Heart Rate Measurement"),
        (0x2ADE, "Mesh Proxy Data Out"),
    ],
)
def test_uuid16(uuid: int, name: str) -> None:
    """Test the characteristic dict with known values."""
    assert characteristic[uuid] == name


@pytest.mark.parametrize(
    "uuid",
    [
        -1,
        65536,
        6.5,
    ],
)
def test_invalid_uuid16(uuid: int) -> None:
    """Test the characteristic dict with invalid keys.

    Using a key that's not a 16-bit unsigned integer should raise a
    No16BitIntegerError exception.
    """
    with pytest.raises(No16BitIntegerError):
        _ = characteristic[uuid]


@pytest.mark.parametrize(
    "uuid",
    [
        0x0000,
        0x2ACA,
        0xFFFF,
    ],
)
def test_unknown_uuid16(uuid: int) -> None:
    """Test the characteristic dict with unknown 16-bit UUIDs.

    Using an unknown 16-bit UUID as a key should raise an UnknownUUIDError
    exception.
    """
    with pytest.raises(UnknownUUIDError):
        _ = characteristic[uuid]


@pytest.mark.parametrize(
    ("uuid", "name"),
    [
        (UUID("00001524-1212-EFDE-1523-785FEABCD123"), "Blinky Button State"),
        (UUID("ef680301-9B35-4933-9b10-52ffa9740042"), "Thingy LED State"),
        (UUID("E95D1B25-251D-470A-A062-FA1922DFA9A8"), "micro:bit Temperature Period"),
    ],
)
def test_uuid128(uuid: UUID, name: str) -> None:
    """Test the characteristic dict with 128-bit non-standard UUIDs."""
    assert characteristic[uuid] == name


@pytest.mark.parametrize(
    ("uuid", "name"),
    [
        (
            UUID("00002A7E-0000-1000-8000-00805F9B34FB"),
            "Aerobic Heart Rate Lower Limit",
        ),
        (UUID("00002a37-0000-1000-8000-00805f9B34fb"), "Heart Rate Measurement"),
        (UUID("00002ADE-0000-1000-8000-00805F9B34FB"), "Mesh Proxy Data Out"),
    ],
)
def test_uuid16_as_uuid128(uuid: UUID, name: str) -> None:
    """Test the characteristic dict with 128-bit standard UUIDs."""
    assert characteristic[uuid] == name


@pytest.mark.parametrize(
    "uuid",
    [
        UUID("2ac35699-9af0-4228-80fb-8ca56c77ac73"),
        UUID("9652BA73-AA7C-4B56-BB93-CCC6D44937C8"),
    ],
)
def test_unknown_uuid128(uuid: UUID) -> None:
    """Test the characteristic dict with unknown 128-bit UUIDs.

    Using an unknown 128-bit UUID as a key should raise an UnknownUUIDError
    exception.
    """
    with pytest.raises(UnknownUUIDError):
        _ = characteristic[uuid]



"""Test the bluetooth_numbers._services module."""
from uuid import UUID

import pytest

from bluetooth_numbers import service
from bluetooth_numbers.exceptions import No16BitIntegerError, UnknownUUIDError


@pytest.mark.parametrize(
    ("uuid", "name"),
    [
        (0x1800, "Generic Access"),
        (0x1812, "Human Interface Device"),
        (0xFD6F, "Exposure Notification Service"),
    ],
)
def test_uuid16(uuid: int, name: str) -> None:
    """Test the service dict with known 16-bit UUIDs."""
    assert service[uuid] == name


@pytest.mark.parametrize(
    "uuid",
    [
        -1,
        65536,
        6.5,
    ],
)
def test_invalid_uuid16(uuid: int) -> None:
    """Test the service dict with invalid keys.

    Using a key that's not a 16-bit unsigned integer should raise a
    No16BitIntegerError exception.
    """
    with pytest.raises(No16BitIntegerError):
        _ = service[uuid]


@pytest.mark.parametrize(
    "uuid",
    [
        0x0000,
        0x1799,
        0xFFFF,
    ],
)
def test_unknown_uuid16(uuid: int) -> None:
    """Test the service dict with unknown 16-bit UUIDs.

    Using an unknown 16-bit UUID as a key should raise an UnknownUUIDError
    exception.
    """
    with pytest.raises(UnknownUUIDError):
        _ = service[uuid]


@pytest.mark.parametrize(
    ("uuid", "name"),
    [
        (
            UUID("7905F431-B5CE-4E99-A40F-4B1E122D00D0"),
            "Apple Notification Center Service",
        ),
        (UUID("6e400001-b5a3-f393-e0a9-e50e24dcca9e"), "Nordic UART Service"),
        (UUID("8D53DC1D-1DB7-4CD3-868B-8A527460AA84"), "SMP Service"),
    ],
)
def test_uuid128(uuid: UUID, name: str) -> None:
    """Test the service dict with 128-bit non-standard UUIDs."""
    assert service[uuid] == name


@pytest.mark.parametrize(
    ("uuid", "name"),
    [
        (UUID("00001800-0000-1000-8000-00805F9B34FB"), "Generic Access"),
        (UUID("00001812-0000-1000-8000-00805F9B34FB"), "Human Interface Device"),
        (UUID("0000FD6F-0000-1000-8000-00805F9B34FB"), "Exposure Notification Service"),
    ],
)
def test_uuid16_as_uuid128(uuid: UUID, name: str) -> None:
    """Test the service dict with 128-bit standard UUIDs."""
    assert service[uuid] == name


@pytest.mark.parametrize(
    "uuid",
    [
        UUID("e85e7f31-69a0-4784-ae25-fd3f452bf563"),
        UUID("85AB7C38-2C67-4A4F-8379-E6BC606D15EA"),
    ],
)
def test_unknown_uuid128(uuid: UUID) -> None:
    """Test the service dict with unknown 128-bit UUIDs.

    Using an unknown 128-bit UUID as a key should raise an UnknownUUIDError
    exception.
    """
    with pytest.raises(UnknownUUIDError):
        _ = service[uuid]



"""Module with UUIDs and names for Bluetooth services.

Usage:

>>> from bluetooth_numbers import service
>>> from uuid import UUID
>>> service[0x180F]
'Battery Service'
>>> service[UUID("6E400001-B5A3-F393-E0A9-E50E24DCCA9E")]
'Nordic UART Service'
"""
from uuid import UUID

from bluetooth_numbers.dicts import UUIDDict

service = UUIDDict(
    {
        0x1800: "Generic Access",
        0x1811: "Alert Notification Service",
        0x1815: "Automation IO",
        0x180F: "Battery Service",
        0x1810: "Blood Pressure",
        0x181B: "Body Composition",
        0x181E: "Bond Management Service",
        0x181F: "Continuous Glucose Monitoring",
        0x1805: "Current Time Service",
        0x1818: "Cycling Power",
        0x1816: "Cycling Speed and Cadence",
        0x180A: "Device Information",
        0x181A: "Environmental Sensing",
        0x1826: "Fitness Machine",
        0x1801: "Generic Attribute",
        0x1808: "Glucose",
        0x1809: "Health Thermometer",
        0x180D: "Heart Rate",
        0x1823: "HTTP Proxy",
        0x1812: "Human Interface Device",
        0x1802: "Immediate Alert",
        0x1821: "Indoor Positioning",
        0x183A: "Insulin Delivery",
        0x1820: "Internet Protocol Support Service",
        0x1803: "Link Loss",
        0x1819: "Location and Navigation",
        0x1827: "Mesh Provisioning Service",
        0x1828: "Mesh Proxy Service",
        0x1807: "Next DST Change Service",
        0x1825: "Object Transfer Service",
        0x180E: "Phone Alert Status Service",
        0x1822: "Pulse Oximeter Service",
        0x1829: "Reconnection Configuration",
        0x1806: "Reference Time Update Service",
        0x1814: "Running Speed and Cadence",
        0x1813: "Scan Parameters",
        0x1824: "Transport Discovery",
        0x1804: "Tx Power",
        0x181C: "User Data",
        0x181D: "Weight Scale",
        0x183B: "Binary Sensor",
        0x183C: "Emergency Configuration",
        0x183E: "Physical Activity Monitor",
        0x1843: "Audio Input Control",
        0x1844: "Volume Control",
        0x1845: "Volume Offset Control",
        0x1846: "Coordinated Set Identification",
        0x1847: "Device Time",
        0x1848: "Media Control",
        0x1849: "Generic Media Control",
        0x184A: "Constant Tone Extension",
        0x184B: "Telephone Bearer",
        0x184C: "Generic Telephone Bearer",
        0x184D: "Microphone Control",
        0x184E: "Audio Stream Control",
        0x184F: "Broadcast Audio Scan",
        0x1850: "Published Audio Capabilities",
        0x1851: "Basic Audio Announcement",
        0x1852: "Broadcast Audio Announcement",
        0x1853: "Common Audio",
        0x1854: "Hearing Access",
        0x1855: "TMAS",
        0xFE0F: "Signify Netherlands B.V. (formerly Philips Lighting) Service",
        0xFEAA: "Eddystone",
        0xFE2C: "Fast Pair Service",
        0xFE59: "Secure DFU Service",
        0xFD6F: "Exposure Notification Service",
        0xFEBB: "File Transfer Service by Adafruit",
        0xFD0D: "Blecon Advertising Service",
        0xFCB7: "T-Mobile USA",
        0xFCB8: "Ribbiot, INC.",
        0xFCB9: "Lumi United Technology Co., Ltd",
        0xFCBA: "BlueID GmbH",
        0xFCBB: "SharkNinja Operating LLC",
        0xFCBC: "Drowsy Digital, Inc.",
        0xFCBD: "Toshiba Corporation",
        0xFCBE: "Musen Connect, Inc.",
        0xFCBF: "ASSA ABLOY Opening Solutions Sweden AB",
        0xFCC0: "Xiaomi Inc.",
        0xFCC1: "TIMECODE SYSTEMS LIMITED",
        0xFCC2: "Qualcomm Technologies, Inc.",
        0xFCC3: "HP Inc.",
        0xFCC4: "OMRON(DALIAN) CO,.LTD.",
        0xFCC5: "OMRON(DALIAN) CO,.LTD.",
        0xFCC6: "Wiliot LTD.",
        0xFCC7: "PB INC.",
        0xFCC8: "Allthenticate, Inc.",
        0xFCC9: "SkyHawke Technologies",
        0xFCCA: "Cosmed s.r.l.",
        0xFCCB: "TOTO LTD.",
        0xFCCC: "WiFi Alliance",
        0xFCCD: "Zound Industries International AB",
        0xFCCE: "Luna Health, Inc.",
        0xFCCF: "Google LLC",
        0xFCD0: "Laerdal Medical AS",
        0xFCD1: "Shenzhen Benwei Media Co.,Ltd.",
        0xFCD2: "Allterco Robotics ltd",
        0xFCD3: "Fisher & Paykel Healthcare",
        0xFCD4: "OMRON HEALTHCARE",
        0xFCD5: "Nortek Security & Control",
        0xFCD6: "SWISSINNO SOLUTIONS AG",
        0xFCD7: "PowerPal Pty Ltd",
        0xFCD8: "Appex Factory S.L.",
        0xFCD9: "Huso, INC",
        0xFCDA: "Draeger",
        0xFCDB: "aconno GmbH",
        0xFCDC: "Amazon.com Services, LLC",
        0xFCDD: "Mobilaris AB",
        0xFCDE: "ARCTOP, INC.",
        0xFCDF: "NIO USA, Inc.",
        0xFCE0: "Akciju sabiedriba ”SAF TEHNIKA”",
        0xFCE1: "Sony Group Corporation",
        0xFCE2: "Baracoda Daily Healthtech",
        0xFCE3: "Smith & Nephew Medical Limited",
        0xFCE4: "Samsara Networks, Inc",
        0xFCE5: "Samsara Networks, Inc",
        0xFCE6: "Guard RFID Solutions Inc.",
        0xFCE7: "TKH Security B.V.",
        0xFCE8: "ITT Industries",
        0xFCE9: "MindRhythm, Inc.",
        0xFCEA: "Chess Wise B.V.",
        0xFCEB: "Avi­On",
        0xFCEC: "Griffwerk GmbH",
        0xFCED: "Workaround Gmbh",
        0xFCEE: "Velentium, LLC",
        0xFCEF: "Divesoft s.r.o.",
        0xFCF0: "Security Enhancement Systems, LLC",
        0xFCF1: "Google LLC",
        0xFCF2: "Bitwards Oy",
        0xFCF3: "Armatura LLC",
        0xFCF4: "Allegion",
        0xFCF5: "Trident Communication Technology, LLC",
        0xFCF6: "The Linux Foundation",
        0xFCF7: "Honor Device Co., Ltd.",
        0xFCF8: "Honor Device Co., Ltd.",
        0xFCF9: "Leupold & Stevens, Inc.",
        0xFCFA: "Leupold & Stevens, Inc.",
        0xFCFB: "Shenzhen Benwei Media Co., Ltd.",
        0xFCFC: "Barrot Technology Limited",
        0xFCFD: "Barrot Technology Limited",
        0xFCFE: "Sennheiser Consumer Audio GmbH",
        0xFCFF: "701x",
        0xFD00: "FUTEK Advanced Sensor Technology, Inc.",
        0xFD01: "Sanvita Medical Corporation",
        0xFD02: "LEGO System A/S",
        0xFD03: "Quuppa Oy",
        0xFD04: "Shure Inc.",
        0xFD05: "Qualcomm Technologies, Inc.",
        0xFD06: "RACE­AI LLC",
        0xFD07: "Swedlock AB",
        0xFD08: "Bull Group Incorporated Company",
        0xFD09: "Cousins and Sears LLC",
        0xFD0A: "Luminostics, Inc.",
        0xFD0B: "Luminostics, Inc.",
        0xFD0C: "OSM HK Limited",
        0xFD0E: "HerdDogg, Inc",
        0xFD0F: "AEON MOTOR CO.,LTD.",
        0xFD10: "AEON MOTOR CO.,LTD.",
        0xFD11: "AEON MOTOR CO.,LTD.",
        0xFD12: "AEON MOTOR CO.,LTD.",
        0xFD13: "BRG Sports, Inc.",
        0xFD14: "BRG Sports, Inc.",
        0xFD15: "Panasonic Corporation",
        0xFD16: "Sensitech, Inc.",
        0xFD17: "LEGIC Identsystems AG",
        0xFD18: "LEGIC Identsystems AG",
        0xFD19: "Smith & Nephew Medical Limited",
        0xFD1A: "CSIRO",
        0xFD1B: "Helios Sports, Inc.",
        0xFD1C: "Brady Worldwide Inc.",
        0xFD1D: "Samsung Electronics Co., Ltd",
        0xFD1E: "Plume Design Inc.",
        0xFD1F: "3M",
        0xFD20: "GN Hearing A/S",
        0xFD21: "Huawei Technologies Co., Ltd.",
        0xFD22: "Huawei Technologies Co., Ltd.",
        0xFD23: "DOM Sicherheitstechnik GmbH & Co. KG",
        0xFD24: "GD Midea Air­Conditioning Equipment Co., Ltd.",
        0xFD25: "GD Midea Air­Conditioning Equipment Co., Ltd.",
        0xFD26: "Novo Nordisk A/S",
        0xFD27: "i2Systems",
        0xFD28: "Julius Blum GmbH",
        0xFD29: "Asahi Kasei Corporation",
        0xFD2A: "Sony Corporation",
        0xFD2B: "The Access Technologies",
        0xFD2C: "The Access Technologies",
        0xFD2D: "Xiaomi Inc.",
        0xFD2E: "Bitstrata Systems Inc.",
        0xFD2F: "Bitstrata Systems Inc.",
        0xFD30: "Sesam Solutions BV",
        0xFD31: "LG Electronics Inc.",
        0xFD32: "Gemalto Holding BV",
        0xFD33: "DashLogic, Inc.",
        0xFD34: "Aerosens LLC.",
        0xFD35: "Transsion Holdings Limited",
        0xFD36: "Google LLC",
        0xFD37: "TireCheck GmbH",
        0xFD38: "Danfoss A/S",
        0xFD39: "PREDIKTAS",
        0xFD3A: "Verkada Inc.",
        0xFD3B: "Verkada Inc.",
        0xFD3C: "Redline Communications Inc.",
        0xFD3D: "Woan Technology (Shenzhen) Co., Ltd.",
        0xFD3E: "Pure Watercraft, inc.",
        0xFD3F: "Cognosos, Inc",
        0xFD40: "Beflex Inc.",
        0xFD41: "Amazon Lab126",
        0xFD42: "Globe (Jiangsu) Co.,Ltd",
        0xFD43: "Apple Inc.",
        0xFD44: "Apple Inc.",
        0xFD45: "GB Solution co.,Ltd",
        0xFD46: "Lemco IKE",
        0xFD47: "Liberty Global Inc.",
        0xFD48: "Geberit International AG",
        0xFD49: "Panasonic Corporation",
        0xFD4A: "Sigma Elektro GmbH",
        0xFD4B: "Samsung Electronics Co., Ltd.",
        0xFD4C: "Adolf Wuerth GmbH & Co KG",
        0xFD4D: "70mai Co.,Ltd.",
        0xFD4E: "70mai Co.,Ltd.",
        0xFD4F: "Forkbeard Technologies AS",
        0xFD50: "Hangzhou Tuya Information Technology Co., Ltd",
        0xFD51: "UTC Fire and Security",
        0xFD52: "UTC Fire and Security",
        0xFD53: "PCI Private Limited",
        0xFD54: "Qingdao Haier Technology Co., Ltd.",
        0xFD55: "Braveheart Wireless, Inc.",
        0xFD56: "Resmed Ltd",
        0xFD57: "Volvo Car Corporation",
        0xFD58: "Volvo Car Corporation",
        0xFD59: "Samsung Electronics Co., Ltd.",
        0xFD5A: "Samsung Electronics Co., Ltd.",
        0xFD5B: "V2SOFT INC.",
        0xFD5C: "React Mobile",
        0xFD5D: "maxon motor ltd.",
        0xFD5E: "Tapkey GmbH",
        0xFD5F: "Oculus VR, LLC",
        0xFD60: "Sercomm Corporation",
        0xFD61: "Arendi AG",
        0xFD62: "Fitbit, Inc.",
        0xFD63: "Fitbit, Inc.",
        0xFD64: "INRIA",
        0xFD65: "Razer Inc.",
        0xFD66: "Zebra Technologies Corporation",
        0xFD67: "Montblanc Simplo GmbH",
        0xFD68: "Ubique Innovation AG",
        0xFD69: "Samsung Electronics Co., Ltd",
        0xFD6A: "Emerson",
        0xFD6B: "rapitag GmbH",
        0xFD6C: "Samsung Electronics Co., Ltd.",
        0xFD6D: "Sigma Elektro GmbH",
        0xFD6E: "Polidea sp. z o.o.",
        0xFD70: "GuangDong Oppo Mobile Telecommunications Corp., Ltd",
        0xFD71: "GN Hearing A/S",
        0xFD72: "Logitech International SA",
        0xFD73: "BRControls Products BV",
        0xFD74: "BRControls Products BV",
        0xFD75: "Insulet Corporation",
        0xFD76: "Insulet Corporation",
        0xFD77: "Withings",
        0xFD78: "Withings",
        0xFD79: "Withings",
        0xFD7A: "Withings",
        0xFD7B: "WYZE LABS, INC.",
        0xFD7C: "Toshiba Information Systems(Japan) Corporation",
        0xFD7D: "Center for Advanced Research Wernher Von Braun",
        0xFD7E: "Samsung Electronics Co., Ltd.",
        0xFD7F: "Husqvarna AB",
        0xFD80: "Phindex Technologies, Inc",
        0xFD81: "CANDY HOUSE, Inc.",
        0xFD82: "Sony Corporation",
        0xFD83: "iNFORM Technology GmbH",
        0xFD84: "Tile, Inc.",
        0xFD85: "Husqvarna AB",
        0xFD86: "Abbott",
        0xFD87: "Google LLC",
        0xFD88: "Urbanminded LTD",
        0xFD89: "Urbanminded LTD",
        0xFD8A: "Signify Netherlands B.V.",
        0xFD8B: "Jigowatts Inc.",
        0xFD8C: "Google LLC",
        0xFD8D: "quip NYC Inc.",
        0xFD8E: "Motorola Solutions",
        0xFD8F: "Matrix ComSec Pvt. Ltd.",
        0xFD90: "Guangzhou SuperSound Information Technology Co.,Ltd",
        0xFD91: "Groove X, Inc.",
        0xFD92: "Qualcomm Technologies International, Ltd. (QTIL)",
        0xFD93: "Bayerische Motoren Werke AG",
        0xFD94: "Hewlett Packard Enterprise",
        0xFD95: "Rigado",
        0xFD96: "Google LLC",
        0xFD97: "June Life, Inc.",
        0xFD98: "Disney Worldwide Services, Inc.",
        0xFD99: "ABB Oy",
        0xFD9A: "Huawei Technologies Co., Ltd.",
        0xFD9B: "Huawei Technologies Co., Ltd.",
        0xFD9C: "Huawei Technologies Co., Ltd.",
        0xFD9D: "Gastec Corporation",
        0xFD9E: "The Coca­Cola Company",
        0xFD9F: "VitalTech Affiliates LLC",
        0xFDA0: "Secugen Corporation",
        0xFDA1: "Groove X, Inc",
        0xFDA2: "Groove X, Inc",
        0xFDA3: "Inseego Corp.",
        0xFDA4: "Inseego Corp.",
        0xFDA5: "Neurostim OAB, Inc.",
        0xFDA6: "WWZN Information Technology Company Limited",
        0xFDA7: "WWZN Information Technology Company Limited",
        0xFDA8: "PSA Peugeot Citroën",
        0xFDA9: "Rhombus Systems, Inc.",
        0xFDAA: "Xiaomi Inc.",
        0xFDAB: "Xiaomi Inc.",
        0xFDAC: "Tentacle Sync GmbH",
        0xFDAD: "Houwa System Design, k.k.",
        0xFDAE: "Houwa System Design, k.k.",
        0xFDAF: "Wiliot LTD",
        0xFDB0: "Proxy Technologies, Inc.",
        0xFDB1: "Proxy Technologies, Inc.",
        0xFDB2: "Portable Multimedia Ltd",
        0xFDB3: "Audiodo AB",
        0xFDB4: "HP Inc",
        0xFDB5: "ECSG",
        0xFDB6: "GWA Hygiene GmbH",
        0xFDB7: "LivaNova USA Inc.",
        0xFDB8: "LivaNova USA Inc.",
        0xFDB9: "Comcast Cable Corporation",
        0xFDBA: "Comcast Cable Corporation",
        0xFDBB: "Profoto",
        0xFDBC: "Emerson",
        0xFDBD: "Clover Network, Inc.",
        0xFDBE: "California Things Inc.",
        0xFDBF: "California Things Inc.",
        0xFDC0: "Hunter Douglas",
        0xFDC1: "Hunter Douglas",
        0xFDC2: "Baidu Online Network Technology (Beijing) Co., Ltd",
        0xFDC3: "Baidu Online Network Technology (Beijing) Co., Ltd",
        0xFDC4: "Simavita (Aust) Pty Ltd",
        0xFDC5: "Automatic Labs",
        0xFDC6: "Eli Lilly and Company",
        0xFDC7: "Eli Lilly and Company",
        0xFDC8: "Hach - Danaher",
        0xFDC9: "Busch­Jaeger Elektro GmbH",
        0xFDCA: "Fortin Electronic Systems",
        0xFDCB: "Meggitt SA",
        0xFDCC: "Shoof Technologies",
        0xFDCD: "Qingping Technology (Beijing) Co., Ltd.",
        0xFDCE: "SENNHEISER electronic GmbH & Co. KG",
        0xFDCF: "Nalu Medical, Inc",
        0xFDD0: "Huawei Technologies Co., Ltd",
        0xFDD1: "Huawei Technologies Co., Ltd",
        0xFDD2: "Bose Corporation",
        0xFDD3: "FUBA Automotive Electronics GmbH",
        0xFDD4: "LX Solutions Pty Limited",
        0xFDD5: "Brompton Bicycle Ltd",
        0xFDD6: "Ministry of Supply",
        0xFDD7: "Emerson",
        0xFDD8: "Jiangsu Teranovo Tech Co., Ltd.",
        0xFDD9: "Jiangsu Teranovo Tech Co., Ltd.",
        0xFDDA: "MHCS",
        0xFDDB: "Samsung Electronics Co., Ltd.",
        0xFDDC: "4iiii Innovations Inc.",
        0xFDDD: "Arch Systems Inc",
        0xFDDE: "Noodle Technology Inc.",
        0xFDDF: "Harman International",
        0xFDE0: "John Deere",
        0xFDE1: "Fortin Electronic Systems",
        0xFDE2: "Google LLC",
        0xFDE3: "Abbott Diabetes Care",
        0xFDE4: "JUUL Labs, Inc.",
        0xFDE5: "SMK Corporation",
        0xFDE6: "Intelletto Technologies Inc",
        0xFDE7: "SECOM Co., LTD",
        0xFDE8: "Robert Bosch GmbH",
        0xFDE9: "Spacesaver Corporation",
        0xFDEA: "SeeScan, Inc",
        0xFDEB: "Syntronix Corporation",
        0xFDEC: "Mannkind Corporation",
        0xFDED: "Pole Star",
        0xFDEE: "Huawei Technologies Co., Ltd.",
        0xFDEF: "ART AND PROGRAM, INC.",
        0xFDF0: "Google LLC",
        0xFDF1: "LAMPLIGHT Co.,Ltd",
        0xFDF2: "AMICCOM Electronics Corporation",
        0xFDF3: "Amersports",
        0xFDF4: "O. E. M. Controls, Inc.",
        0xFDF5: "Milwaukee Electric Tools",
        0xFDF6: "AIAIAI ApS",
        0xFDF7: "HP Inc.",
        0xFDF8: "Onvocal",
        0xFDF9: "INIA",
        0xFDFA: "Tandem Diabetes Care",
        0xFDFB: "Tandem Diabetes Care",
        0xFDFC: "Optrel AG",
        0xFDFD: "RecursiveSoft Inc.",
        0xFDFE: "ADHERIUM(NZ) LIMITED",
        0xFDFF: "OSRAM GmbH",
        0xFE00: "Amazon.com Services, Inc.",
        0xFE01: "Duracell U.S. Operations Inc.",
        0xFE02: "Robert Bosch GmbH",
        0xFE03: "Amazon.com Services, Inc.",
        0xFE04: "OpenPath Security Inc",
        0xFE05: "CORE Transport Technologies NZ Limited",
        0xFE06: "Qualcomm Technologies, Inc.",
        0xFE07: "Sonos, Inc.",
        0xFE08: "Microsoft",
        0xFE09: "Pillsy, Inc.",
        0xFE0A: "ruwido austria gmbh",
        0xFE0B: "ruwido austria gmbh",
        0xFE0C: "Procter & Gamble",
        0xFE0D: "Procter & Gamble",
        0xFE0E: "Setec Pty Ltd",
        0xFE10: "LAPIS Technology Co., Ltd.",
        0xFE11: "GMC­I Messtechnik GmbH",
        0xFE12: "M­Way Solutions GmbH",
        0xFE13: "Apple Inc.",
        0xFE14: "Flextronics International USA Inc.",
        0xFE15: "Amazon.com Services, Inc..",
        0xFE16: "Footmarks, Inc.",
        0xFE17: "Telit Wireless Solutions GmbH",
        0xFE18: "Runtime, Inc.",
        0xFE19: "Google LLC",
        0xFE1A: "Tyto Life LLC",
        0xFE1B: "Tyto Life LLC",
        0xFE1C: "NetMedia, Inc.",
        0xFE1D: "Illuminati Instrument Corporation",
        0xFE1E: "Smart Innovations Co., Ltd",
        0xFE1F: "Garmin International, Inc.",
        0xFE20: "Emerson",
        0xFE21: "Bose Corporation",
        0xFE22: "Zoll Medical Corporation",
        0xFE23: "Zoll Medical Corporation",
        0xFE24: "August Home Inc",
        0xFE25: "Apple, Inc.",
        0xFE26: "Google LLC",
        0xFE27: "Google LLC",
        0xFE28: "Ayla Networks",
        0xFE29: "Gibson Innovations",
        0xFE2A: "DaisyWorks, Inc.",
        0xFE2B: "ITT Industries",
        0xFE2D: "SMART INNOVATION Co.,Ltd",
        0xFE2E: "ERi,Inc.",
        0xFE2F: "CRESCO Wireless, Inc",
        0xFE30: "Volkswagen AG",
        0xFE31: "Volkswagen AG",
        0xFE32: "Pro­Mark, Inc.",
        0xFE33: "CHIPOLO d.o.o.",
        0xFE34: "SmallLoop LLC",
        0xFE35: "HUAWEI Technologies Co., Ltd",
        0xFE36: "HUAWEI Technologies Co., Ltd",
        0xFE37: "Spaceek LTD",
        0xFE38: "Spaceek LTD",
        0xFE39: "TTS Tooltechnic Systems AG & Co. KG",
        0xFE3A: "TTS Tooltechnic Systems AG & Co. KG",
        0xFE3B: "Dolby Laboratories",
        0xFE3C: "alibaba",
        0xFE3D: "BD Medical",
        0xFE3E: "BD Medical",
        0xFE3F: "Friday Labs Limited",
        0xFE40: "Inugo Systems Limited",
        0xFE41: "Inugo Systems Limited",
        0xFE42: "Nets A/S",
        0xFE43: "Andreas Stihl AG & Co. KG",
        0xFE44: "SK Telecom",
        0xFE45: "Snapchat Inc",
        0xFE46: "B&O Play A/S",
        0xFE47: "General Motors",
        0xFE48: "General Motors",
        0xFE49: "SenionLab AB",
        0xFE4A: "OMRON HEALTHCARE Co., Ltd.",
        0xFE4B: "Signify Netherlands B.V. (formerly Philips Lighting B.V.)",
        0xFE4C: "Volkswagen AG",
        0xFE4D: "Casambi Technologies Oy",
        0xFE4E: "NTT docomo",
        0xFE4F: "Molekule, Inc.",
        0xFE50: "Google LLC",
        0xFE51: "SRAM",
        0xFE52: "SetPoint Medical",
        0xFE53: "3M",
        0xFE54: "Motiv, Inc.",
        0xFE55: "Google LLC",
        0xFE56: "Google LLC",
        0xFE57: "Dotted Labs",
        0xFE58: "Nordic Semiconductor ASA",
        0xFE5A: "Cronologics Corporation",
        0xFE5B: "GT­tronics HK Ltd",
        0xFE5C: "million hunters GmbH",
        0xFE5D: "Grundfos A/S",
        0xFE5E: "Plastc Corporation",
        0xFE5F: "Eyefi, Inc.",
        0xFE60: "Lierda Science & Technology Group Co., Ltd.",
        0xFE61: "Logitech International SA",
        0xFE62: "Indagem Tech LLC",
        0xFE63: "Connected Yard, Inc.",
        0xFE64: "Siemens AG",
        0xFE65: "CHIPOLO d.o.o.",
        0xFE66: "Intel Corporation",
        0xFE67: "Lab Sensor Solutions",
        0xFE68: "Qualcomm Life Inc",
        0xFE69: "Qualcomm Life Inc",
        0xFE6A: "Kontakt Micro­Location Sp. z o.o.",
        0xFE6B: "TASER International, Inc.",
        0xFE6C: "TASER International, Inc.",
        0xFE6D: "The University of Tokyo",
        0xFE6E: "The University of Tokyo",
        0xFE6F: "LINE Corporation",
        0xFE70: "Beijing Jingdong Century Trading Co., Ltd.",
        0xFE71: "Plume Design Inc",
        0xFE72: "Abbott (formerly St. Jude Medical, Inc.)",
        0xFE73: "Abbott (formerly St. Jude Medical, Inc.)",
        0xFE74: "unwire",
        0xFE75: "TangoMe",
        0xFE76: "TangoMe",
        0xFE77: "Hewlett­Packard Company",
        0xFE78: "Hewlett­Packard Company",
        0xFE79: "Zebra Technologies",
        0xFE7A: "Bragi GmbH",
        0xFE7B: "Orion Labs, Inc.",
        0xFE7C: "Telit Wireless Solutions (Formerly Stollmann E+V GmbH)",
        0xFE7D: "Aterica Health Inc.",
        0xFE7E: "Awear Solutions Ltd",
        0xFE7F: "Doppler Lab",
        0xFE80: "Doppler Lab",
        0xFE81: "Medtronic Inc.",
        0xFE82: "Medtronic Inc.",
        0xFE83: "Blue Bite",
        0xFE84: "RF Digital Corp",
        0xFE85: "RF Digital Corp",
        0xFE86: "HUAWEI Technologies Co., Ltd",
        0xFE87: "Qingdao Yeelink Information Technology Co., Ltd. ( 青岛亿联客信息技术有限公司 )",
        0xFE88: "SALTO SYSTEMS S.L.",
        0xFE89: "B&O Play A/S",
        0xFE8A: "Apple, Inc.",
        0xFE8B: "Apple, Inc.",
        0xFE8C: "TRON Forum",
        0xFE8D: "Interaxon Inc.",
        0xFE8E: "ARM Ltd",
        0xFE8F: "CSR",
        0xFE90: "JUMA",
        0xFE91: "Shanghai Imilab Technology Co.,Ltd",
        0xFE92: "Jarden Safety & Security",
        0xFE93: "OttoQ In",
        0xFE94: "OttoQ In",
        0xFE95: "Xiaomi Inc.",
        0xFE96: "Tesla Motors Inc.",
        0xFE97: "Tesla Motors Inc.",
        0xFE98: "Currant Inc",
        0xFE99: "Currant Inc",
        0xFE9A: "Estimote",
        0xFE9B: "Samsara Networks, Inc",
        0xFE9C: "GSI Laboratories, Inc.",
        0xFE9D: "Mobiquity Networks Inc",
        0xFE9E: "Dialog Semiconductor B.V.",
        0xFE9F: "Google LLC",
        0xFEA0: "Google LLC",
        0xFEA1: "Intrepid Control Systems, Inc.",
        0xFEA2: "Intrepid Control Systems, Inc.",
        0xFEA3: "ITT Industries",
        0xFEA4: "Paxton Access Ltd",
        0xFEA5: "GoPro, Inc.",
        0xFEA6: "GoPro, Inc.",
        0xFEA7: "UTC Fire and Security",
        0xFEA8: "Savant Systems LLC",
        0xFEA9: "Savant Systems LLC",
        0xFEAB: "Nokia",
        0xFEAC: "Nokia",
        0xFEAD: "Nokia",
        0xFEAE: "Nokia",
        0xFEAF: "Nest Labs Inc",
        0xFEB0: "Nest Labs Inc",
        0xFEB1: "Electronics Tomorrow Limited",
        0xFEB2: "Microsoft Corporation",
        0xFEB3: "Taobao",
        0xFEB4: "WiSilica Inc.",
        0xFEB5: "WiSilica Inc.",
        0xFEB6: "Vencer Co., Ltd",
        0xFEB7: "Meta Platforms, Inc.",
        0xFEB8: "Meta Platforms, Inc.",
        0xFEB9: "LG Electronics",
        0xFEBA: "Tencent Holdings Limited",
        0xFEBC: "Dexcom Inc",
        0xFEBD: "Clover Network, Inc",
        0xFEBE: "Bose Corporation",
        0xFEBF: "Nod, Inc.",
        0xFEC0: "KDDI Corporation",
        0xFEC1: "KDDI Corporation",
        0xFEC2: "Blue Spark Technologies, Inc.",
        0xFEC3: "360fly, Inc.",
        0xFEC4: "PLUS Location Systems",
        0xFEC5: "Realtek Semiconductor Corp.",
        0xFEC6: "Kocomojo, LLC",
        0xFEC7: "Apple, Inc.",
        0xFEC8: "Apple, Inc.",
        0xFEC9: "Apple, Inc.",
        0xFECA: "Apple, Inc.",
        0xFECB: "Apple, Inc.",
        0xFECC: "Apple, Inc.",
        0xFECD: "Apple, Inc.",
        0xFECE: "Apple, Inc.",
        0xFECF: "Apple, Inc.",
        0xFED0: "Apple, Inc.",
        0xFED1: "Apple, Inc.",
        0xFED2: "Apple, Inc.",
        0xFED3: "Apple, Inc.",
        0xFED4: "Apple, Inc.",
        0xFED5: "Plantronics Inc.",
        0xFED6: "Broadcom",
        0xFED7: "Broadcom",
        0xFED8: "Google LLC",
        0xFED9: "Pebble Technology Corporation",
        0xFEDA: "ISSC Technologies Corp.",
        0xFEDB: "Perka, Inc.",
        0xFEDC: "Jawbone",
        0xFEDD: "Jawbone",
        0xFEDE: "Coin, Inc.",
        0xFEDF: "Design SHIFT",
        0xFEE0: "Anhui Huami Information Technology Co., Ltd.",
        0xFEE1: "Anhui Huami Information Technology Co., Ltd.",
        0xFEE2: "Anki, Inc.",
        0xFEE3: "Anki, Inc.",
        0xFEE4: "Nordic Semiconductor ASA",
        0xFEE5: "Nordic Semiconductor ASA",
        0xFEE6: "Silvair, Inc.",
        0xFEE7: "Tencent Holdings Limited.",
        0xFEE8: "Quintic Corp.",
        0xFEE9: "Quintic Corp.",
        0xFEEA: "Swirl Networks, Inc.",
        0xFEEB: "Swirl Networks, Inc.",
        0xFEEC: "Tile, Inc.",
        0xFEED: "Tile, Inc.",
        0xFEEE: "Polar Electro Oy",
        0xFEEF: "Polar Electro Oy",
        0xFEF0: "Intel",
        0xFEF1: "CSR",
        0xFEF2: "CSR",
        0xFEF3: "Google LLC",
        0xFEF4: "Google LLC",
        0xFEF5: "Dialog Semiconductor GmbH",
        0xFEF6: "Wicentric, Inc.",
        0xFEF7: "Aplix Corporation",
        0xFEF8: "Aplix Corporation",
        0xFEF9: "PayPal, Inc.",
        0xFEFA: "PayPal, Inc.",
        0xFEFB: "Telit Wireless Solutions (Formerly Stollmann E+V GmbH)",
        0xFEFC: "Gimbal, Inc.",
        0xFEFD: "Gimbal, Inc.",
        0xFEFE: "GN ReSound A/S",
        0xFEFF: "GN Netcom",
        0xFFF3: "FiRa Consortium service",
        0xFFF4: "FiRa Consortium service",
        0xFFF5: "Car Connectivity Consortium, LLC service",
        0xFFF6: "ZigBee Alliance service",
        0xFFF7: "ZigBee Alliance service",
        0xFFF8: "Mopria Alliance BLE Service service",
        0xFFF9: "FIDO2 secure client-to-authenticator transport service",
        0xFFFA: "ASTM Remote ID service",
        0xFFFB: "Direct Thread Commissioning service",
        0xFFFC: "Wireless Power Transfer (WPT) Service service",
        0xFFFD: "Universal Second Factor Authenticator Service service",
        0xFFFE: "Wireless Power Transfer Service service",
        UUID(
            "932C32BD-0000-47A2-835A-A8D455B859DD",
        ): "Philips Hue Light Control Service",
        UUID(
            "B8843ADD-0000-4AA1-8794-C3F462030BDA",
        ): "Philips Hue Light Update Service",
        UUID(
            "7905F431-B5CE-4E99-A40F-4B1E122D00D0",
        ): "Apple Notification Center Service",
        UUID("89D3502B-0F36-433A-8EF4-C502AD55F8DC"): "Apple Media Service",
        UUID("7DFC6000-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Service",
        UUID("7DFC7000-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Service",
        UUID("7DFC8000-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Service",
        UUID("7DFC9000-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Service",
        UUID("E95D0753-251D-470A-A062-FA1922DFA9A8"): "micro:bit Accelerometer Service",
        UUID("E95DF2D8-251D-470A-A062-FA1922DFA9A8"): "micro:bit Magnetometer Service",
        UUID("E95D9882-251D-470A-A062-FA1922DFA9A8"): "micro:bit Button Service",
        UUID("E95D127B-251D-470A-A062-FA1922DFA9A8"): "micro:bit IO Pin Service",
        UUID("E95DD91D-251D-470A-A062-FA1922DFA9A8"): "micro:bit LED Service",
        UUID("E95D93AF-251D-470A-A062-FA1922DFA9A8"): "micro:bit Event Service",
        UUID("E95D93B0-251D-470A-A062-FA1922DFA9A8"): "micro:bit DFU Control Service",
        UUID("E95D6100-251D-470A-A062-FA1922DFA9A8"): "micro:bit Temperature Service",
        UUID("EF680100-9B35-4933-9B10-52FFA9740042"): "Thingy Configuration Service",
        UUID("EF680200-9B35-4933-9B10-52FFA9740042"): "Thingy Weather Station Service",
        UUID("EF680300-9B35-4933-9B10-52FFA9740042"): "Thingy UI Service",
        UUID("EF680400-9B35-4933-9B10-52FFA9740042"): "Thingy Motion Service",
        UUID("EF680500-9B35-4933-9B10-52FFA9740042"): "Thingy Sound Service",
        UUID("00001523-1212-EFDE-1523-785FEABCD123"): "Nordic LED and Button Service",
        UUID("6E400001-B5A3-F393-E0A9-E50E24DCCA9E"): "Nordic UART Service",
        UUID("57A70000-9350-11ED-A1EB-0242AC120002"): "Nordic Status Message Service",
        UUID("A3C87500-8ED3-4BDF-8A39-A01BEBEDE295"): "Eddystone Configuration Service",
        UUID("00001530-1212-EFDE-1523-785FEABCD123"): "Legacy DFU Service",
        UUID(
            "8E400001-F315-4F60-9FB8-838830DAEA50",
        ): "Experimental Buttonless DFU Service",
        UUID(
            "E2A00001-EC31-4EC3-A97A-1C34D87E9878",
        ): "Edge Impulse Remote Management Service",
        UUID("8D53DC1D-1DB7-4CD3-868B-8A527460AA84"): "SMP Service",
        UUID(
            "00001623-1212-EFDE-1623-785FEABCD123",
        ): "LEGO® Wireless Protocol v3 Hub Service",
        UUID(
            "00001625-1212-EFDE-1623-785FEABCD123",
        ): "LEGO® Wireless Protocol v3 Bootloader Service",
        UUID("ADAF0100-C332-42A8-93BD-25E905756CB8"): "Adafruit Temperature Service",
        UUID("ADAF0200-C332-42A8-93BD-25E905756CB8"): "Adafruit Accelerometer Service",
        UUID("ADAF0300-C332-42A8-93BD-25E905756CB8"): "Adafruit Light Service",
        UUID("ADAF0400-C332-42A8-93BD-25E905756CB8"): "Adafruit Gyroscope Service",
        UUID("ADAF0500-C332-42A8-93BD-25E905756CB8"): "Adafruit Magnetometer Service",
        UUID("ADAF0600-C332-42A8-93BD-25E905756CB8"): "Adafruit Button Service",
        UUID("ADAF0700-C332-42A8-93BD-25E905756CB8"): "Adafruit Humidity Service",
        UUID("ADAF0800-C332-42A8-93BD-25E905756CB8"): "Adafruit Barometric Service",
        UUID("ADAF0900-C332-42A8-93BD-25E905756CB8"): "Adafruit Addressable Service",
        UUID("ADAF0A00-C332-42A8-93BD-25E905756CB8"): "Adafruit Color Service",
        UUID("ADAF0B00-C332-42A8-93BD-25E905756CB8"): "Adafruit Sound Service",
        UUID("ADAF0C00-C332-42A8-93BD-25E905756CB8"): "Adafruit Tone Service",
        UUID("ADAF0D00-C332-42A8-93BD-25E905756CB8"): "Adafruit Quaternion Service",
        UUID("ADAF0E00-C332-42A8-93BD-25E905756CB8"): "Adafruit Proximity Service",
        UUID(
            "F000FFC0-0451-4000-B000-000000000000",
        ): "Texas Instruments Over-the-Air Download (OAD) Service",
        UUID("0FDA92B2-44A2-4AF2-84F5-FA682BAA2B8D"): "Helium Hotspot Custom Service",
        UUID("54220000-F6A5-4007-A371-722F4EBD8436"): "Memfault Diagnostic Service",
    },
)



"""Module with specialized dictionary classes for UUIDs, CICs and OUIs."""
from __future__ import annotations

from typing import Dict, Union
from uuid import UUID

from bluetooth_numbers.exceptions import (
    No16BitIntegerError,
    NonStandardUUIDError,
    UnknownCICError,
    UnknownOUIError,
    UnknownUUIDError,
)
from bluetooth_numbers.utils import (
    is_normalized_oui,
    is_uint16,
    normalize_oui,
    uuid128_to_uuid16,
)


class CICDict(Dict[int, str]):
    """Dictionary class to hold 16-bit company codes and their names.

    You can use this class as a dict with the following differences:

    - If you check for a key that doesn't exist, this raises an
      :class:`~bluetooth_numbers.exceptions.UnknownCICError`.
    - If you check for a key that isn't a 16-bit unsigned integer, this raises a
      :class:`~bluetooth_numbers.exceptions.No16BitIntegerError`.

    Examples:
        >>> from bluetooth_numbers import company
        >>> company[0x004C]
        'Apple, Inc.'
        >>> company[-1]
        Traceback (most recent call last):
        bluetooth_numbers.exceptions.No16BitIntegerError: -1
        >>> company[65534]
        Traceback (most recent call last):
        bluetooth_numbers.exceptions.UnknownCICError: 65534
    """

    def __missing__(self, key: int) -> str:
        """Try the key and raise exception when it's invalid.

        Args:
            key (int): The key to check.

        Raises:
            No16BitIntegerError: If ``key`` isn't a 16-bit unsigned integer.
            UnknownCICError: If ``key`` isn't in this CICDict instance.
        """
        if is_uint16(key):
            raise UnknownCICError(key)

        raise No16BitIntegerError(key)


class OUIDict(Dict[str, str]):
    """Dictionary class to hold OUIs and their names.

    You can use this class as a dict with the following differences:

    - You can check for an OUI in the formats "xx:yy:zz", "xx-yy-zz" or "xxyyzz".
      Both lowercase and uppercase letters are supported.
    - If you check for a key that doesn't exist, this raises an
      :class:`~bluetooth_numbers.exceptions.UnknownOUIError`.
    - If you check for a key that doesn't have one of the supported formats, this
      raises a :class:`~bluetooth_numbers.exceptions.WrongOUIFormatError`.

    Examples:
        >>> from bluetooth_numbers import oui
        >>> oui["98:E7:43"]
        'Dell Inc.'
        >>> oui["c4-29-96"]
        'Signify B.V.'
        >>> oui["A44519"]
        'Xiaomi Communications Co Ltd'
        >>> oui["FOOBAR"]
        Traceback (most recent call last):
        bluetooth_numbers.exceptions.WrongOUIFormatError: 'FOOBAR'
        >>> oui["AB:CD:EF"]
        Traceback (most recent call last):
        bluetooth_numbers.exceptions.UnknownOUIError: AB:CD:EF
    """

    def __missing__(self, key: str) -> str:
        """Try the key and raise exception when it's invalid.

        Args:
            key (str): The key to check.

        Raises:
            UnknownOUIError: If ``key`` isn't in this OUIDict instance.
            WrongOUIFormatError: If ``key`` doesn't have one of the supported
              formats.

        Returns:
            str: The name corresponding to ``key``.
        """
        if is_normalized_oui(key):
            raise UnknownOUIError(key)

        return self[normalize_oui(key)]


class UUIDDict(Dict[Union[UUID, int], str]):
    """Dictionary class to hold 16-bit and 128-bit standard UUID keys and descriptions.

    You can use this class as a dict for Bluetooth UUIDs, with the following
    differences:

    - If you check for a 128-bit standard UUID and this UUID doesn't exist in the
      dictionary, it will check for the corresponding 16-bit UUID.
    - If you check for a UUID that doesn't exist, this raises an
      :class:`~bluetooth_numbers.exceptions.UnknownUUIDError`.
    - If you check for a key that isn't a 16-bit unsigned integer, this raises a
      :class:`~bluetooth_numbers.exceptions.No16BitIntegerError`.

    Examples:
        >>> from bluetooth_numbers import service
        >>> from uuid import UUID
        >>> service[UUID("0000180F-0000-1000-8000-00805F9B34FB")]
        'Battery Service'
        >>> service[0x180F]
        'Battery Service'
        >>> service[0]
        Traceback (most recent call last):
        bluetooth_numbers.exceptions.UnknownUUIDError: 0
        >>> service[6.5]
        Traceback (most recent call last):
        bluetooth_numbers.exceptions.No16BitIntegerError: 6.5
    """

    def __missing__(self, key: UUID | int) -> str:
        """Try the key converted to 16-bit UUID.

        Args:
            key (UUID | int): The 128-bit or 16-bit UUID to check.

        Raises:
            No16BitIntegerError: If ``key`` isn't a 16-bit unsigned integer.
            UnknownUUIDError: If ``key`` isn't in this UUIDDict instance.

        Returns:
            str: The name corresponding to ``key``.
        """
        if isinstance(key, UUID):
            try:
                return self[uuid128_to_uuid16(key)]
            except NonStandardUUIDError as error:
                raise UnknownUUIDError(key) from error
        elif is_uint16(key):
            raise UnknownUUIDError(key)

        raise No16BitIntegerError(key)



"""Test the bluetooth_numbers._descriptors module."""
import pytest

from bluetooth_numbers import descriptor
from bluetooth_numbers.exceptions import No16BitIntegerError, UnknownUUIDError


@pytest.mark.parametrize(
    ("uuid", "name"),
    [
        (0x2900, "Characteristic Extended Properties"),
        (0x2906, "Valid Range"),
        (0x290E, "Time Trigger Setting"),
    ],
)
def test_uuid16(uuid: int, name: str) -> None:
    """Test the descriptor dict with known values."""
    assert descriptor[uuid] == name


@pytest.mark.parametrize(
    "uuid",
    [
        -1,
        65536,
        6.5,
    ],
)
def test_invalid_uuid16(uuid: int) -> None:
    """Test the descriptor dict with invalid keys.

    Using a key that's not a 16-bit unsigned integer should raise a
    No16BitIntegerError exception.
    """
    with pytest.raises(No16BitIntegerError):
        _ = descriptor[uuid]


@pytest.mark.parametrize(
    "uuid",
    [
        0x0000,
        0x28FF,
        0xFFFF,
    ],
)
def test_unknown_uuid16(uuid: int) -> None:
    """Test the descriptor dict with unknown 16-bit UUIDs.

    Using an unknown 16-bit UUID as a key should raise an UnknownUUIDError
    exception.
    """
    with pytest.raises(UnknownUUIDError):
        _ = descriptor[uuid]



# pylint: disable=line-too-long
"""Test the bluetooth_numbers._companies module."""
import pytest

from bluetooth_numbers import company
from bluetooth_numbers.exceptions import No16BitIntegerError, UnknownCICError


@pytest.mark.parametrize(
    ("code", "name"),
    [
        (0x0000, "Ericsson AB"),
        (0x004C, "Apple, Inc."),
        (0x0499, "Ruuvi Innovations Ltd."),
        (
            0xFFFF,
            "Bluetooth SIG Specification Reserved Default Vendor ID for Remote Devices Without Device ID Service Record.",  # noqa: E501
        ),
    ],
)
def test_company(code: int, name: str) -> None:
    """Test the company dict with known values."""
    assert company[code] == name


@pytest.mark.parametrize(
    "code",
    [
        -1,
        65536,
        6.5,
        "test",
    ],
)
def test_invalid_company(code: int) -> None:
    """Test the company dict with invalid keys.

    Using a key that's not a 16-bit unsigned integer should raise a
    No16BitIntegerError exception.
    """
    with pytest.raises(No16BitIntegerError):
        _ = company[code]


@pytest.mark.parametrize(
    "code",
    [
        0xEEEE,
        65534,
    ],
)
def test_unknown_company(code: int) -> None:
    """Test the company dict with unknown keys.

    Using an unknown company code as a key should raise an UnknownCICError exception.
    """
    with pytest.raises(UnknownCICError):
        _ = company[code]



"""Module with exceptions raised by this library."""


class BluetoothNumbersError(Exception):
    """Base class for all exceptions raised by this library."""


class No16BitIntegerError(BluetoothNumbersError):
    """Exception raised when an integer is not a 16-bit number."""


class NonStandardUUIDError(BluetoothNumbersError):
    """Exception raised when a 128-bit UUID is not a standard Bluetooth UUID."""


class UnknownCICError(BluetoothNumbersError):
    """Exception raised when a CIC is not known."""


class UnknownOUIError(BluetoothNumbersError):
    """Exception raised when an OUI is not known."""


class UnknownUUIDError(BluetoothNumbersError):
    """Exception raised when a UUID is not known."""


class WrongOUIFormatError(BluetoothNumbersError):
    """Exception raised when a string isn't a supported format for an OUI."""



# pylint: skip-file
"""bluetooth-numbers: Python package with a wide set of numbers related to Bluetooth.

This project offers a Python package with company IDs, service, characteristic and
descriptor UUIDs and OUIs, so Python projects can easily use these numbers.

Get the description of a company ID:

>>> from bluetooth_numbers import company
>>> company[0x0499]
'Ruuvi Innovations Ltd.'

Get the description of a service UUID:

>>> from bluetooth_numbers import service
>>> from uuid import UUID
>>> service[0x180F]
'Battery Service'
>>> service[UUID("6E400001-B5A3-F393-E0A9-E50E24DCCA9E")]
'Nordic UART Service'

Get the description of a characteristic UUID:

>>> from bluetooth_numbers import characteristic
>>> from uuid import UUID
>>> characteristic[0x2A37]
'Heart Rate Measurement'
>>> characteristic[UUID("6E400002-B5A3-F393-E0A9-E50E24DCCA9E")]
'UART RX Characteristic'

Get the description of a descriptor UUID:

>>> from bluetooth_numbers import descriptor
>>> descriptor[0x2901]
'Characteristic User Descriptor'

Get the description of an OUI:

>>> from bluetooth_numbers import oui
>>> oui["58:2D:34"]
'Qingping Electronics (Suzhou) Co., Ltd'
"""
import sys

# Public API for easier importing
from ._characteristics import characteristic
from ._companies import company
from ._descriptors import descriptor
from ._ouis import oui
from ._services import service

__all__ = ["characteristic", "company", "descriptor", "oui", "service"]

if sys.version_info[:2] >= (3, 8):
    # TODO: Import directly (no need for conditional) when `python_requires = >= 3.8`
    from importlib.metadata import PackageNotFoundError, version  # pragma: no cover
else:
    from importlib_metadata import PackageNotFoundError, version  # pragma: no cover

try:
    # Change here if project is renamed and does not equal the package name
    dist_name = "bluetooth-numbers"
    __version__ = version(dist_name)
except PackageNotFoundError:  # pragma: no cover
    __version__ = "unknown"
finally:
    del version, PackageNotFoundError



"""Module with UUIDs and names for Bluetooth characteristics.

Usage:

>>> from bluetooth_numbers import characteristic
>>> from uuid import UUID
>>> characteristic[0x2A37]
'Heart Rate Measurement'
>>> characteristic[UUID("6E400002-B5A3-F393-E0A9-E50E24DCCA9E")]
'UART RX Characteristic'
"""
from uuid import UUID

from bluetooth_numbers.dicts import UUIDDict

characteristic = UUIDDict(
    {
        0x2A7E: "Aerobic Heart Rate Lower Limit",
        0x2A84: "Aerobic Heart Rate Upper Limit",
        0x2A7F: "Aerobic Threshold",
        0x2A80: "Age",
        0x2A5A: "Aggregate",
        0x2A43: "Alert Category ID",
        0x2A42: "Alert Category ID Bit Mask",
        0x2A06: "Alert Level",
        0x2A44: "Alert Notification Control Point",
        0x2A3F: "Alert Status",
        0x2AB3: "Altitude",
        0x2A81: "Anaerobic Heart Rate Lower Limit",
        0x2A82: "Anaerobic Heart Rate Upper Limit",
        0x2A83: "Anaerobic Threshold",
        0x2A58: "Analog",
        0x2A59: "Analog Output",
        0x2A73: "Apparent Wind Direction",
        0x2A72: "Apparent Wind Speed",
        0x2A01: "Appearance",
        0x2AA3: "Barometric Pressure Trend",
        0x2A19: "Battery Level",
        0x2A1B: "Battery Level State",
        0x2A1A: "Battery Power State",
        0x2A49: "Blood Pressure Feature",
        0x2A35: "Blood Pressure Measurement",
        0x2A9B: "Body Composition Feature",
        0x2A9C: "Body Composition Measurement",
        0x2A38: "Body Sensor Location",
        0x2AA4: "Bond Management Control Point",
        0x2AA5: "Bond Management Features",
        0x2A22: "Boot Keyboard Input Report",
        0x2A32: "Boot Keyboard Output Report",
        0x2A33: "Boot Mouse Input Report",
        0x2AA6: "Central Address Resolution",
        0x2AA8: "CGM Feature",
        0x2AA7: "CGM Measurement",
        0x2AAB: "CGM Session Run Time",
        0x2AAA: "CGM Session Start Time",
        0x2AAC: "CGM Specific Ops Control Point",
        0x2AA9: "CGM Status",
        0x2ACE: "Cross Trainer Data",
        0x2A5C: "CSC Feature",
        0x2A5B: "CSC Measurement",
        0x2A2B: "Current Time",
        0x2A66: "Cycling Power Control Point",
        0x2A65: "Cycling Power Feature",
        0x2A63: "Cycling Power Measurement",
        0x2A64: "Cycling Power Vector",
        0x2A99: "Database Change Increment",
        0x2A85: "Date of Birth",
        0x2A86: "Date of Threshold Assessment",
        0x2A08: "Date Time",
        0x2A0A: "Day Date Time",
        0x2A09: "Day of Week",
        0x2A7D: "Descriptor Value Changed",
        0x2A00: "Device Name",
        0x2A7B: "Dew Point",
        0x2A56: "Digital",
        0x2A57: "Digital Output",
        0x2A0D: "DST Offset",
        0x2A6C: "Elevation",
        0x2A87: "Email Address",
        0x2A0B: "Exact Time 100",
        0x2A0C: "Exact Time 256",
        0x2A88: "Fat Burn Heart Rate Lower Limit",
        0x2A89: "Fat Burn Heart Rate Upper Limit",
        0x2A26: "Firmware Revision String",
        0x2A8A: "First Name",
        0x2AD9: "Fitness Machine Control Point",
        0x2ACC: "Fitness Machine Feature",
        0x2ADA: "Fitness Machine Status",
        0x2A8B: "Five Zone Heart Rate Limits",
        0x2AB2: "Floor Number",
        0x2A8C: "Gender",
        0x2A51: "Glucose Feature",
        0x2A18: "Glucose Measurement",
        0x2A34: "Glucose Measurement Context",
        0x2A74: "Gust Factor",
        0x2A27: "Hardware Revision String",
        0x2A39: "Heart Rate Control Point",
        0x2A8D: "Heart Rate Max",
        0x2A37: "Heart Rate Measurement",
        0x2A7A: "Heat Index",
        0x2A8E: "Height",
        0x2A4C: "HID Control Point",
        0x2A4A: "HID Information",
        0x2A8F: "Hip Circumference",
        0x2ABA: "HTTP Control Point",
        0x2AB9: "HTTP Entity Body",
        0x2AB7: "HTTP Headers",
        0x2AB8: "HTTP Status Code",
        0x2ABB: "HTTPS Security",
        0x2A6F: "Humidity",
        0x2B22: "IDD Annunciation Status",
        0x2B25: "IDD Command Control Point",
        0x2B26: "IDD Command Data",
        0x2B23: "IDD Features",
        0x2B28: "IDD History Data",
        0x2B27: "IDD Record Access Control Point",
        0x2B21: "IDD Status",
        0x2B20: "IDD Status Changed",
        0x2B24: "IDD Status Reader Control Point",
        0x2A2A: "IEEE 11073-20601 Regulatory Certification Data List",
        0x2AD2: "Indoor Bike Data",
        0x2AAD: "Indoor Positioning Configuration",
        0x2A36: "Intermediate Cuff Pressure",
        0x2A1E: "Intermediate Temperature",
        0x2A77: "Irradiance",
        0x2AA2: "Language",
        0x2A90: "Last Name",
        0x2AAE: "Latitude",
        0x2A6B: "LN Control Point",
        0x2A6A: "LN Feature",
        0x2AB1: "Local East Coordinate",
        0x2AB0: "Local North Coordinate",
        0x2A0F: "Local Time Information",
        0x2A67: "Location and Speed Characteristic",
        0x2AB5: "Location Name",
        0x2AAF: "Longitude",
        0x2A2C: "Magnetic Declination",
        0x2AA0: "Magnetic Flux Density - 2D",
        0x2AA1: "Magnetic Flux Density - 3D",
        0x2A29: "Manufacturer Name String",
        0x2A91: "Maximum Recommended Heart Rate",
        0x2A21: "Measurement Interval",
        0x2A24: "Model Number String",
        0x2A68: "Navigation",
        0x2A3E: "Network Availability",
        0x2A46: "New Alert",
        0x2AC5: "Object Action Control Point",
        0x2AC8: "Object Changed",
        0x2AC1: "Object First-Created",
        0x2AC3: "Object ID",
        0x2AC2: "Object Last-Modified",
        0x2AC6: "Object List Control Point",
        0x2AC7: "Object List Filter",
        0x2ABE: "Object Name",
        0x2AC4: "Object Properties",
        0x2AC0: "Object Size",
        0x2ABF: "Object Type",
        0x2ABD: "OTS Feature",
        0x2A04: "Peripheral Preferred Connection Parameters",
        0x2A02: "Peripheral Privacy Flag",
        0x2A5F: "PLX Continuous Measurement Characteristic",
        0x2A60: "PLX Features",
        0x2A5E: "PLX Spot-Check Measurement",
        0x2A50: "PnP ID",
        0x2A75: "Pollen Concentration",
        0x2A2F: "Position 2D",
        0x2A30: "Position 3D",
        0x2A69: "Position Quality",
        0x2A6D: "Pressure",
        0x2A4E: "Protocol Mode",
        0x2A62: "Pulse Oximetry Control Point",
        0x2A78: "Rainfall",
        0x2B1D: "RC Feature",
        0x2B1E: "RC Settings",
        0x2A03: "Reconnection Address",
        0x2B1F: "Reconnection Configuration Control Point",
        0x2A52: "Record Access Control Point",
        0x2A14: "Reference Time Information",
        0x2A3A: "Removable",
        0x2A4D: "Report",
        0x2A4B: "Report Map",
        0x2AC9: "Resolvable Private Address Only",
        0x2A92: "Resting Heart Rate",
        0x2A40: "Ringer Control point",
        0x2A41: "Ringer Setting",
        0x2AD1: "Rower Data",
        0x2A54: "RSC Feature",
        0x2A53: "RSC Measurement",
        0x2A55: "SC Control Point",
        0x2A4F: "Scan Interval Window",
        0x2A31: "Scan Refresh",
        0x2A3C: "Scientific Temperature Celsius",
        0x2A10: "Secondary Time Zone",
        0x2A5D: "Sensor Location",
        0x2A05: "Service Changed",
        0x2A0E: "Time Zone",
        0x2A11: "Time with DST",
        0x2A12: "Time Accuracy",
        0x2A13: "Time Source",
        0x2A15: "Time Broadcast",
        0x2A16: "Time Update Control Point",
        0x2A17: "Time Update State",
        0x2A25: "Serial Number String",
        0x2A3B: "Service Required",
        0x2A28: "Software Revision String",
        0x2A93: "Sport Type for Aerobic and Anaerobic Thresholds",
        0x2AD0: "Stair Climber Data",
        0x2ACF: "Step Climber Data",
        0x2A3D: "String",
        0x2AD7: "Supported Heart Rate Range",
        0x2AD5: "Supported Inclination Range",
        0x2A47: "Supported New Alert Category",
        0x2AD8: "Supported Power Range",
        0x2AD6: "Supported Resistance Level Range",
        0x2AD4: "Supported Speed Range",
        0x2A48: "Supported Unread Alert Category",
        0x2A23: "System ID",
        0x2ABC: "TDS Control Point",
        0x2A6E: "Temperature",
        0x2A1F: "Temperature Celsius",
        0x2A20: "Temperature Fahrenheit",
        0x2A1C: "Temperature Measurement",
        0x2A1D: "Temperature Type",
        0x2A94: "Three Zone Heart Rate Limits",
        0x2A70: "True Wind Speed",
        0x2A71: "True Wind Direction",
        0x2A95: "Two Zone Heart Rate Limit",
        0x2A07: "Tx Power Level",
        0x2AB4: "Uncertainty",
        0x2A45: "Unread Alert Status",
        0x2AB6: "URI",
        0x2A9F: "User Control Point",
        0x2A9A: "User Index",
        0x2A76: "UV Index",
        0x2A79: "Wind Chill",
        0x2A96: "VO2 Max",
        0x2A97: "Waist Circumference",
        0x2A98: "Weight",
        0x2A9D: "Weight Measurement",
        0x2A9E: "Weight Scale Feature",
        0x2ACD: "Treadmill Data",
        0x2AD3: "Training Status",
        0x2AE0: "Average Current",
        0x2AE1: "Average Voltage",
        0x2AE2: "Boolean",
        0x2AE3: "Chromatic Distance From Planckian",
        0x2AE4: "Chromaticity Coordinates",
        0x2AE5: "Chromaticity In CCT And Duv Values",
        0x2AE6: "Chromaticity Tolerance",
        0x2AE7: "CIE 13.3-1995 Color Rendering Index",
        0x2AE8: "Coefficient",
        0x2AE9: "Correlated Color Temperature",
        0x2AEA: "Count 16",
        0x2AEB: "Count 24",
        0x2AEC: "Country Code",
        0x2AED: "Date UTC",
        0x2AEE: "Electric Current",
        0x2AEF: "Electric Current Range",
        0x2AF0: "Electric Current Specification",
        0x2AF1: "Electric Current Statistics",
        0x2AF2: "Energy",
        0x2AF3: "Energy In A Period Of Day",
        0x2AF4: "Event Statistics",
        0x2AF5: "Fixed String 16",
        0x2AF6: "Fixed String 24",
        0x2AF7: "Fixed String 36",
        0x2AF8: "Fixed String 8",
        0x2AF9: "Generic Level",
        0x2AFA: "Global Trade Item Number",
        0x2AFB: "Illuminance",
        0x2AFC: "Luminous Efficacy",
        0x2AFD: "Luminous Energy",
        0x2AFE: "Luminous Exposure",
        0x2AFF: "Luminous Flux",
        0x2B00: "Luminous Flux Range",
        0x2B01: "Luminous Intensity",
        0x2B02: "B02 Mass Flow",
        0x2B03: "Perceived Lightness",
        0x2B04: "Percentage 8",
        0x2B05: "Power",
        0x2B06: "Power Specification",
        0x2B07: "Relative Runtime In A Current Range",
        0x2B08: "Relative Runtime In A Generic Level Range",
        0x2B09: "Relative Value In A Voltage Range",
        0x2B0A: "Relative Value In An Illuminance Range",
        0x2B0B: "Relative Value In A Period Of Day",
        0x2B0C: "Relative Value In A Temperature Range",
        0x2B0D: "Temperature 8",
        0x2B0E: "Temperature 8 In A Period Of Day",
        0x2B0F: "Temperature 8 Statistics",
        0x2B10: "Temperature Range",
        0x2B11: "Temperature Statistics",
        0x2B12: "Time Decihour 8",
        0x2B13: "Time Exponential 8",
        0x2B14: "Time Hour 24",
        0x2B15: "Time Millisecond 24",
        0x2B16: "Time Second 16",
        0x2B17: "Time Second 8",
        0x2B18: "Voltage",
        0x2B19: "Voltage Specification",
        0x2B1A: "Voltage Statistics",
        0x2B1B: "Volume Flow",
        0x2B1C: "Chromaticity Coordinate",
        0x2B29: "Client Supported Features",
        0x2B2A: "Database Hash",
        0x2B2B: "BSS Control Point",
        0x2B2C: "BSS Response",
        0x2B2D: "Emergency ID",
        0x2B2E: "Emergency Text",
        0x2B34: "Enhanced Blood Pressure Measurement",
        0x2B35: "Enhanced Intermediate Cuff Pressure",
        0x2B36: "Blood Pressure Record",
        0x2B38: "BR-EDR Handover Data",
        0x2B39: "Bluetooth SIG Data",
        0x2B3A: "Server Supported Features",
        0x2B3B: "Physical Activity Monitor Features",
        0x2B3C: "General Activity Instantaneous Data",
        0x2B3D: "General Activity Summary Data",
        0x2B3E: "CardioRespiratory Activity Instantaneous Data",
        0x2B3F: "CardioRespiratory Activity Summary Data",
        0x2B40: "Step Counter Activity Summary Data",
        0x2B41: "Sleep Activity Instantaneous Data",
        0x2B42: "Sleep Activity Summary Data",
        0x2B43: "Physical Activity Monitor Control Point",
        0x2B44: "Activity Current Session",
        0x2B45: "Physical Activity Session Descriptor",
        0x2B46: "Preferred Units",
        0x2B47: "High Resolution Height",
        0x2B48: "Middle Name",
        0x2B49: "Stride Length",
        0x2B4A: "Handedness",
        0x2B4B: "Device Wearing Position",
        0x2B4C: "Four Zone Heart Rate Limits",
        0x2B4D: "High Intensity Exercise Threshold",
        0x2B4E: "Activity Goal",
        0x2B4F: "Sedentary Interval Notification",
        0x2B50: "Caloric Intake",
        0x2B51: "TMAP Role",
        0x2B77: "Audio Input State",
        0x2B78: "Gain Settings Attribute",
        0x2B79: "Audio Input Type",
        0x2B7A: "Audio Input Status",
        0x2B7B: "Audio Input Control Point",
        0x2B7C: "Audio Input Description",
        0x2B7D: "Volume State",
        0x2B7E: "Volume Control Point",
        0x2B7F: "Volume Flags",
        0x2B80: "Volume Offset State",
        0x2B81: "Audio Location",
        0x2B82: "Volume Offset Control Point",
        0x2B83: "Audio Output Description",
        0x2B84: "Set Identity Resolving Key",
        0x2B85: "Coordinated Set Size",
        0x2B86: "Set Member Lock",
        0x2B87: "Set Member Rank",
        0x2B8E: "Device Time Feature",
        0x2B8F: "Device Time Parameters",
        0x2B90: "Device Time",
        0x2B91: "Device Time Control Point",
        0x2B92: "Time Change Log Data",
        0x2B93: "Media Player Name",
        0x2B94: "Media Player Icon Object ID",
        0x2B95: "Media Player Icon URL",
        0x2B96: "Track Changed",
        0x2B97: "Track Title",
        0x2B98: "Track Duration",
        0x2B99: "Track Position",
        0x2B9A: "Playback Speed",
        0x2B9B: "Seeking Speed",
        0x2B9C: "Current Track Segments Object ID",
        0x2B9D: "Current Track Object ID",
        0x2B9E: "Next Track Object ID",
        0x2B9F: "Parent Group Object ID",
        0x2BA0: "Current Group Object ID",
        0x2BA1: "Playing Order",
        0x2BA2: "Playing Orders Supported",
        0x2BA3: "Media State",
        0x2BA4: "Media Control Point",
        0x2BA5: "Media Control Point Opcodes Supported",
        0x2BA6: "Search Results Object ID",
        0x2BA7: "Search Control Point",
        0x2BA9: "Media Player Icon Object Type",
        0x2BAA: "Track Segments Object Type",
        0x2BAB: "Track Object Type",
        0x2BAC: "Group Object Type",
        0x2BAD: "Constant Tone Extension Enable",
        0x2BAE: "Advertising Constant Tone Extension Minimum Length",
        0x2BAF: "Advertising Constant Tone Extension Minimum Transmit Count",
        0x2BB0: "Advertising Constant Tone Extension Transmit Duration",
        0x2BB1: "Advertising Constant Tone Extension Interval",
        0x2BB2: "Advertising Constant Tone Extension PHY",
        0x2BB3: "Bearer Provider Name",
        0x2BB4: "Bearer UCI",
        0x2BB5: "Bearer Technology",
        0x2BB6: "Bearer URI Schemes Supported List",
        0x2BB7: "Bearer Signal Strength",
        0x2BB8: "Bearer Signal Strength Reporting Interval",
        0x2BB9: "Bearer List Current Calls",
        0x2BBA: "Content Control ID",
        0x2BBB: "Status Flags",
        0x2BBC: "Incoming Call Target Bearer URI",
        0x2BBD: "Call State",
        0x2BBE: "Call Control Point",
        0x2BBF: "Call Control Point Optional Opcodes",
        0x2BC0: "Termination Reason",
        0x2BC1: "Incoming Call",
        0x2BC2: "Call Friendly Name",
        0x2BC3: "Mute",
        0x2BC4: "Sink ASE",
        0x2BC5: "Source ASE",
        0x2BC6: "ASE Control Point",
        0x2BC7: "Broadcast Audio Scan Control Point",
        0x2BC8: "Broadcast Receive State",
        0x2BC9: "Sink PAC",
        0x2BCA: "Sink Audio Locations",
        0x2BCB: "Source PAC",
        0x2BCC: "Source Audio Locations",
        0x2BCD: "Available Audio Contexts",
        0x2BCE: "Supported Audio Contexts",
        0x2BCF: "Ammonia Concentration",
        0x2BD0: "Carbon Monoxide Concentration",
        0x2BD1: "Methane Concentration",
        0x2BD2: "Nitrogen Dioxide Concentration",
        0x2BD3: "Non-Methane Volatile Organic Compounds Concentration",
        0x2BD4: "Ozone Concentration",
        0x2BD5: "Particulate Matter - PM1 Concentration",
        0x2BD6: "Particulate Matter - PM2.5 Concentration",
        0x2BD7: "Particulate Matter - PM10 Concentration",
        0x2BD8: "Sulfur Dioxide Concentration",
        0x2BD9: "Sulfur Hexafluoride Concentration",
        0x2BDA: "Hearing Aid Features",
        0x2BDB: "Hearing Aid Preset Control Point",
        0x2BDC: "Active Preset Index",
        0x1233: "Deprecated Fast Pair Model ID",
        0x1234: "Deprecated Fast Pair Key-based Pairing",
        0x1235: "Deprecated Fast Pair Passkey",
        0x1236: "Deprecated Fast Pair Account Key",
        0x1237: "Deprecated Fast Pair Data",
        0x2ADB: "Mesh Provisioning Data In",
        0x2ADC: "Mesh Provisioning Data Out",
        0x2ADD: "Mesh Proxy Data In",
        0x2ADE: "Mesh Proxy Data Out",
        UUID("00001524-1212-EFDE-1523-785FEABCD123"): "Blinky Button State",
        UUID("00001525-1212-EFDE-1523-785FEABCD123"): "Blinky LED State",
        UUID("00001531-1212-EFDE-1523-785FEABCD123"): "Legacy DFU Control Point",
        UUID("00001532-1212-EFDE-1523-785FEABCD123"): "Legacy DFU Packet",
        UUID("00001534-1212-EFDE-1523-785FEABCD123"): "Legacy DFU Version",
        UUID("8EC90001-F315-4F60-9FB8-838830DAEA50"): "DFU Control Point",
        UUID("8EC90002-F315-4F60-9FB8-838830DAEA50"): "DFU Packet",
        UUID("8EC90003-F315-4F60-9FB8-838830DAEA50"): "Buttonless DFU Without Bonds",
        UUID("8EC90004-F315-4F60-9FB8-838830DAEA50"): "Buttonless DFU With Bonds",
        UUID("8E400001-F315-4F60-9FB8-838830DAEA50"): "Experimental Buttonless DFU",
        UUID("DA2E7828-FBCE-4E01-AE9E-261174997C48"): "SMP Characteristic",
        UUID("932C32BD-0002-47A2-835A-A8D455B859DD"): "Philips Hue Light On/Off Toggle",
        UUID(
            "932C32BD-0003-47A2-835A-A8D455B859DD",
        ): "Philips Hue Light Brightness Level",
        UUID("932C32BD-0005-47A2-835A-A8D455B859DD"): "Philips Hue Light Color",
        UUID("EF680101-9B35-4933-9B10-52FFA9740042"): "Thingy Device Name",
        UUID("EF680102-9B35-4933-9B10-52FFA9740042"): "Thingy Advertising Parameters",
        UUID("EF680104-9B35-4933-9B10-52FFA9740042"): "Thingy Connection Parameters",
        UUID("EF680105-9B35-4933-9B10-52FFA9740042"): "Thingy Eddystone URL",
        UUID("EF680106-9B35-4933-9B10-52FFA9740042"): "Thingy Cloud Token",
        UUID("EF680107-9B35-4933-9B10-52FFA9740042"): "Thingy FW Version",
        UUID("EF680108-9B35-4933-9B10-52FFA9740042"): "Thingy MTU Request",
        UUID("EF680201-9B35-4933-9B10-52FFA9740042"): "Thingy Temperature",
        UUID("EF680202-9B35-4933-9B10-52FFA9740042"): "Thingy Pressure",
        UUID("EF680203-9B35-4933-9B10-52FFA9740042"): "Thingy Humidity",
        UUID("EF680204-9B35-4933-9B10-52FFA9740042"): "Thingy Air Quality",
        UUID("EF680205-9B35-4933-9B10-52FFA9740042"): "Thingy Color",
        UUID("EF680206-9B35-4933-9B10-52FFA9740042"): "Thingy Configuration",
        UUID("EF680301-9B35-4933-9B10-52FFA9740042"): "Thingy LED State",
        UUID("EF680302-9B35-4933-9B10-52FFA9740042"): "Thingy Button State",
        UUID("EF680303-9B35-4933-9B10-52FFA9740042"): "Thingy EXT Pin",
        UUID("EF680401-9B35-4933-9B10-52FFA9740042"): "Thingy Motion Config",
        UUID("EF680402-9B35-4933-9B10-52FFA9740042"): "Thingy Tap",
        UUID("EF680403-9B35-4933-9B10-52FFA9740042"): "Thingy Orientation",
        UUID("EF680404-9B35-4933-9B10-52FFA9740042"): "Thingy Quaternion",
        UUID("EF680405-9B35-4933-9B10-52FFA9740042"): "Thingy Pedometer",
        UUID("EF680406-9B35-4933-9B10-52FFA9740042"): "Thingy Raw Data",
        UUID("EF680407-9B35-4933-9B10-52FFA9740042"): "Thingy Euler",
        UUID("EF680408-9B35-4933-9B10-52FFA9740042"): "Thingy Rotation Matrix",
        UUID("EF680409-9B35-4933-9B10-52FFA9740042"): "Thingy Heading",
        UUID("EF68040A-9B35-4933-9B10-52FFA9740042"): "Thingy Gravity Vector",
        UUID("EF680501-9B35-4933-9B10-52FFA9740042"): "Thingy Sound Config",
        UUID("EF680502-9B35-4933-9B10-52FFA9740042"): "Thingy Speaker Data",
        UUID("EF680503-9B35-4933-9B10-52FFA9740042"): "Thingy Speaker Status",
        UUID("EF680504-9B35-4933-9B10-52FFA9740042"): "Thingy Microphone",
        UUID("6E400002-B5A3-F393-E0A9-E50E24DCCA9E"): "UART RX Characteristic",
        UUID("6E400003-B5A3-F393-E0A9-E50E24DCCA9E"): "UART TX Characteristic",
        UUID("57A70001-9350-11ED-A1EB-0242AC120002"): "Status Characteristic",
        UUID(
            "E2A00002-EC31-4EC3-A97A-1C34D87E9878",
        ): "Edge Impulse Remote Management RX Characteristic",
        UUID(
            "E2A00003-EC31-4EC3-A97A-1C34D87E9878",
        ): "Edge Impulse Remote Management TX Characteristic",
        UUID("A3C87501-8ED3-4BDF-8A39-A01BEBEDE295"): "Eddystone Capabilities",
        UUID("A3C87502-8ED3-4BDF-8A39-A01BEBEDE295"): "Eddystone Active Slot",
        UUID("A3C87503-8ED3-4BDF-8A39-A01BEBEDE295"): "Eddystone Advertising Interval",
        UUID("A3C87504-8ED3-4BDF-8A39-A01BEBEDE295"): "Eddystone Radio Tx Power",
        UUID(
            "A3C87505-8ED3-4BDF-8A39-A01BEBEDE295",
        ): "Eddystone (Advanced) Advertised Tx Power",
        UUID("A3C87506-8ED3-4BDF-8A39-A01BEBEDE295"): "Eddystone Lock State",
        UUID("A3C87507-8ED3-4BDF-8A39-A01BEBEDE295"): "Eddystone Unlock",
        UUID("A3C87508-8ED3-4BDF-8A39-A01BEBEDE295"): "Eddystone Public ECDH Key",
        UUID("A3C87509-8ED3-4BDF-8A39-A01BEBEDE295"): "Eddystone EID Identity Key",
        UUID("A3C8750A-8ED3-4BDF-8A39-A01BEBEDE295"): "Eddystone ADV Slot Data",
        UUID(
            "A3C8750B-8ED3-4BDF-8A39-A01BEBEDE295",
        ): "Eddystone Advanced Factory Reset",
        UUID(
            "A3C8750C-8ED3-4BDF-8A39-A01BEBEDE295",
        ): "Eddystone (Advanced) Remain Connectable",
        UUID("FE2C1233-8366-4814-8EB0-01DE32100BEA"): "Fast Pair Model ID",
        UUID("FE2C1234-8366-4814-8EB0-01DE32100BEA"): "Fast Pair Key-based Pairing",
        UUID("FE2C1235-8366-4814-8EB0-01DE32100BEA"): "Fast Pair Passkey",
        UUID("FE2C1236-8366-4814-8EB0-01DE32100BEA"): "Fast Pair Account Key",
        UUID("FE2C1237-8366-4814-8EB0-01DE32100BEA"): "Fast Pair Data",
        UUID("9FBF120D-6301-42D9-8C58-25E699A21DBD"): "Apple Notification Source",
        UUID("69D1D8F3-45E1-49A8-9821-9BBDFDAAD9D9"): "Apple Control Point",
        UUID("22EAC6E9-24D6-4BB5-BE44-B36ACE7C7BFB"): "Apple Data Source",
        UUID("9B3C81D8-57B1-4A8A-B8DF-0E56F7CA51C2"): "Apple Remote Command",
        UUID("2F7CABCE-808D-411F-9A0C-BB92BA96C102"): "Apple Entity Update",
        UUID("C6B2F38C-23AB-46D8-A6AB-A3A870BBD5D7"): "Apple Entity Attribute",
        UUID("7DFC6001-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Characteristic",
        UUID("7DFC6002-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Characteristic",
        UUID("7DFC6003-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Characteristic",
        UUID("7DFC6004-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Characteristic",
        UUID("7DFC6005-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Characteristic",
        UUID("7DFC6101-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Characteristic",
        UUID("7DFC6102-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Characteristic",
        UUID("7DFC6103-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Characteristic",
        UUID("7DFC6104-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Characteristic",
        UUID("7DFC6105-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Characteristic",
        UUID("7DFC6106-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Characteristic",
        UUID("7DFC6107-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Characteristic",
        UUID("7DFC6108-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Characteristic",
        UUID("7DFC6201-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Characteristic",
        UUID("7DFC6202-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Characteristic",
        UUID("7DFC6203-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Characteristic",
        UUID("7DFC8003-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Characteristic",
        UUID("7DFC7004-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Characteristic",
        UUID("7DFC7005-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Characteristic",
        UUID("7DFC7006-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Characteristic",
        UUID("7DFC7007-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Characteristic",
        UUID("7DFC7008-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Characteristic",
        UUID("7DFC7009-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Characteristic",
        UUID("7DFC700A-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Characteristic",
        UUID("7DFC700B-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Characteristic",
        UUID("7DFC700C-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Characteristic",
        UUID("7DFC7103-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Characteristic",
        UUID("7DFC7104-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Characteristic",
        UUID("7DFC7105-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Characteristic",
        UUID("7DFC7106-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Characteristic",
        UUID("7DFC7107-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Characteristic",
        UUID("7DFC7108-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Characteristic",
        UUID("7DFC7109-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Characteristic",
        UUID("7DFC710B-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Characteristic",
        UUID("7DFC710C-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Characteristic",
        UUID("7DFC710D-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Characteristic",
        UUID("7DFC8004-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Characteristic",
        UUID("7DFC9001-7D1C-4951-86AA-8D9728F8D66C"): "Apple Reserved Characteristic",
        UUID("E95DCA4B-251D-470A-A062-FA1922DFA9A8"): "micro:bit Accelerometer Data",
        UUID("E95DFB24-251D-470A-A062-FA1922DFA9A8"): "micro:bit Accelerometer Period",
        UUID("E95DFB11-251D-470A-A062-FA1922DFA9A8"): "micro:bit Magnetometer Data",
        UUID("E95D386C-251D-470A-A062-FA1922DFA9A8"): "micro:bit Magnetometer Period",
        UUID("E95D9715-251D-470A-A062-FA1922DFA9A8"): "micro:bit Magnetometer Bearing",
        UUID("E95DDA90-251D-470A-A062-FA1922DFA9A8"): "micro:bit Button A State",
        UUID("E95DDA91-251D-470A-A062-FA1922DFA9A8"): "micro:bit Button B State",
        UUID("E95D8D00-251D-470A-A062-FA1922DFA9A8"): "micro:bit Pin Data",
        UUID("E95D5899-251D-470A-A062-FA1922DFA9A8"): "micro:bit Pin AD Configuration",
        UUID("E95DB9FE-251D-470A-A062-FA1922DFA9A8"): "micro:bit Pin I/O Configuration",
        UUID("E95DD822-251D-470A-A062-FA1922DFA9A8"): "micro:bit PWM Control",
        UUID("E95D7B77-251D-470A-A062-FA1922DFA9A8"): "micro:bit LED Matrix State",
        UUID("E95D93EE-251D-470A-A062-FA1922DFA9A8"): "micro:bit LED Text",
        UUID("E95D0D2D-251D-470A-A062-FA1922DFA9A8"): "micro:bit Scrolling Delay",
        UUID("E95DB84C-251D-470A-A062-FA1922DFA9A8"): "micro:bit Requirements",
        UUID("E95D9775-251D-470A-A062-FA1922DFA9A8"): "micro:bit Event",
        UUID("E95D23C4-251D-470A-A062-FA1922DFA9A8"): "micro:bit Client Requirements",
        UUID("E95D5404-251D-470A-A062-FA1922DFA9A8"): "micro:bit Client Event",
        UUID("E95D93B1-251D-470A-A062-FA1922DFA9A8"): "micro:bit DFU Control",
        UUID("E95D9250-251D-470A-A062-FA1922DFA9A8"): "micro:bit Temperature",
        UUID("E95D1B25-251D-470A-A062-FA1922DFA9A8"): "micro:bit Temperature Period",
        UUID(
            "00001624-1212-EFDE-1623-785FEABCD123",
        ): "LEGO® Wireless Protocol v3 Hub Characteristic",
        UUID(
            "00001626-1212-EFDE-1623-785FEABCD123",
        ): "LEGO® Wireless Protocol v3 Bootloader Characteristic",
        UUID(
            "ADAF0001-C332-42A8-93BD-25E905756CB8",
        ): "Adafruit Sensor Measurement Period",
        UUID("ADAF0002-C332-42A8-93BD-25E905756CB8"): "Adafruit Sensor Service Version",
        UUID("ADAF0101-C332-42A8-93BD-25E905756CB8"): "Adafruit Temperature",
        UUID("ADAF0201-C332-42A8-93BD-25E905756CB8"): "Adafruit Acceleration",
        UUID("ADAF0301-C332-42A8-93BD-25E905756CB8"): "Adafruit Light Level",
        UUID("ADAF0401-C332-42A8-93BD-25E905756CB8"): "Adafruit Gyro",
        UUID("ADAF0501-C332-42A8-93BD-25E905756CB8"): "Adafruit Magnetic",
        UUID("ADAF0601-C332-42A8-93BD-25E905756CB8"): "Adafruit Pressed",
        UUID("ADAF0701-C332-42A8-93BD-25E905756CB8"): "Adafruit Humidity",
        UUID("ADAF0801-C332-42A8-93BD-25E905756CB8"): "Adafruit Pressure",
        UUID("ADAF0901-C332-42A8-93BD-25E905756CB8"): "Adafruit Pixel Pin",
        UUID("ADAF0902-C332-42A8-93BD-25E905756CB8"): "Adafruit Pixel Pin Type",
        UUID("ADAF0903-C332-42A8-93BD-25E905756CB8"): "Adafruit Pixel Data",
        UUID("ADAF0904-C332-42A8-93BD-25E905756CB8"): "Adafruit Pixel Buffer Size",
        UUID("ADAF0A01-C332-42A8-93BD-25E905756CB8"): "Adafruit Color",
        UUID("ADAF0B01-C332-42A8-93BD-25E905756CB8"): "Adafruit Sound Samples",
        UUID("ADAF0B02-C332-42A8-93BD-25E905756CB8"): "Adafruit Number of Channels",
        UUID("ADAF0C01-C332-42A8-93BD-25E905756CB8"): "Adafruit Tone",
        UUID("ADAF0D01-C332-42A8-93BD-25E905756CB8"): "Adafruit Quaternions",
        UUID("ADAF0D02-C332-42A8-93BD-25E905756CB8"): "Adafruit Calibration In",
        UUID("ADAF0D03-C332-42A8-93BD-25E905756CB8"): "Adafruit Calibration Out",
        UUID("ADAF0E01-C332-42A8-93BD-25E905756CB8"): "Adafruit Proximity",
        UUID("ADAF0100-4669-6C65-5472-616E73666572"): "Adafruit Version",
        UUID("ADAF0200-4669-6C65-5472-616E73666572"): "Adafruit Raw TX/RX",
        UUID(
            "F000FFC1-0451-4000-B000-000000000000",
        ): "Texas Instruments Image Identify",
        UUID("F000FFC2-0451-4000-B000-000000000000"): "Texas Instruments Image Block",
        UUID("F000FFC5-0451-4000-B000-000000000000"): "Texas Instruments OAD Control",
        UUID("D083B2BD-BE16-4600-B397-61512CA2F5AD"): "Helium Hotspot Onboarding Key",
        UUID("0A852C59-50D3-4492-BFD3-22FE58A24F01"): "Helium Hotspot Public Key",
        UUID("D7515033-7E7B-45BE-803F-C8737B171A29"): "Helium Hotspot WiFi Services",
        UUID("B833D34F-D871-422C-BF9E-8E6EC117D57E"): "Helium Hotspot Diagnostics",
        UUID("9C4314F2-8A0C-45FD-A58D-D4A7E64C3A57"): "Helium Hotspot WiFi MAC Address",
        UUID("180EFDEF-7579-4B4A-B2DF-72733B7FA2FE"): "Helium Hotspot Lights",
        UUID("7731DE63-BC6A-4100-8AB1-89B2356B038B"): "Helium Hotspot WiFi SSID",
        UUID("D435F5DE-01A4-4E7D-84BA-DFD347F60275"): "Helium Hotspot Assert Location",
        UUID("DF3B16CA-C985-4DA2-A6D2-9B9B9ABDB858"): "Helium Hotspot Add Gateway",
        UUID("398168AA-0111-4EC0-B1FA-171671270608"): "Helium Hotspot WiFi Connect",
        UUID("E5866BD6-0288-4476-98CA-EF7DA6B4D289"): "Helium Hotspot Ethernet Online",
        UUID("8CC6E0B3-98C5-40CC-B1D8-692940E6994B"): "Helium Hotspot WiFi Remove",
        UUID(
            "E125BDA4-6FB8-11EA-BC55-0242AC130003",
        ): "Helium Hotspot WiFi Configured Services",
        UUID(
            "54220001-F6A5-4007-A371-722F4EBD8436",
        ): "MDS Supported Features Characteristic",
        UUID(
            "54220002-F6A5-4007-A371-722F4EBD8436",
        ): "MDS Device Identifier Characteristic",
        UUID(
            "54220003-F6A5-4007-A371-722F4EBD8436",
        ): "MDS Device Data URI Characteristic",
        UUID(
            "54220004-F6A5-4007-A371-722F4EBD8436",
        ): "MDS Device Authorization Characteristic",
        UUID(
            "54220005-F6A5-4007-A371-722F4EBD8436",
        ): "MDS Device Data Export Characteristic",
    },
)



"""Generate badges with amounts for Bluetooth numbers."""
from pathlib import Path

from bluetooth_numbers import characteristic, company, descriptor, oui, service

README_FILE = "README.rst"
BEFORE_NUMBERS = ".. inclusion-marker-before-numbers\n"
AFTER_NUMBERS = ".. inclusion-marker-after-numbers\n"
REPO = "https://github.com/koenvervloesem/bluetooth-numbers"
SRC_DIR = "blob/main/src/bluetooth_numbers"


def create_badges() -> str:
    """Generate badges with amounts for Bluetooth numbers.

    Returns:
        str: Code for the badges in RST syntax.
    """
    badges = create_badge("Companies", len(company))
    badges += create_badge("Services", len(service))
    badges += create_badge("Characteristics", len(characteristic))
    badges += create_badge("Descriptors", len(descriptor))
    badges += create_badge("OUIs", len(oui))
    return badges


def create_badge(description: str, number: int) -> str:
    """Create custom shields.io badge with a description, number and link.

    Args:
        description (str): Description shown in the badge
        number (int): Amount shown in the badge

    Returns:
        str: Code for the badge in RST syntax
    """
    image = f".. image:: https://img.shields.io/badge/{description}-{number}-blue\n"
    alt = f"    :alt: {description}\n"
    target = f"    :target: {REPO}/{SRC_DIR}/_{description.lower()}.py\n"
    return image + alt + target


if __name__ == "__main__":
    with Path(README_FILE).open(encoding="utf-8") as readme_file:
        readme_text = readme_file.readlines()

    before_numbers = readme_text.index(BEFORE_NUMBERS) + 1
    after_numbers = readme_text.index(AFTER_NUMBERS)

    with Path(README_FILE).open("w", encoding="utf-8") as readme_file:
        readme_file.write("".join(readme_text[:before_numbers]))
        readme_file.write("\n")
        readme_file.write(create_badges())
        readme_file.write("\n")
        readme_file.write("".join(readme_text[after_numbers:]))
